##
# File: TaxonomyExtractor.py
# Date: 15-Oct-2019  jdw
#
# Utilities to extract taxonomy details from the core entity collection.
#
# Updates:
#
##
__docformat__ = "google en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"

import logging

from rcsb.exdb.utils.ObjectExtractor import ObjectExtractor

logger = logging.getLogger(__name__)


class TaxonomyExtractor(object):
    """Utilities to extract taxonomy details from the core entity collection."""

    def __init__(self, cfgOb):
        self.__cfgOb = cfgOb
        self.__databaseName = "pdbx_core"
        self.__collectionName = "pdbx_core_polymer_entity"

    def getUniqueTaxons(self):
        taxIdL = self.__extractEntityTaxons()
        return taxIdL

    def __extractEntityTaxons(self):
        """Test case - extract unique entity source and host taxonomies"""
        try:
            obEx = ObjectExtractor(
                self.__cfgOb,
                databaseName=self.__databaseName,
                collectionName=self.__collectionName,
                cacheFilePath=None,
                useCache=False,
                keyAttribute="entity",
                uniqueAttributes=["rcsb_id"],
                cacheKwargs=None,
                objectLimit=None,
                # selectionQuery={"entity.type": "polymer"},
                selectionQuery=None,
                selectionList=["rcsb_id", "rcsb_entity_source_organism.ncbi_taxonomy_id", "rcsb_entity_host_organism.ncbi_taxonomy_id"],
            )
            eCount = obEx.getCount()
            logger.info("Polymer entity count is %d", eCount)
            taxIdS = set()
            objD = obEx.getObjects()
            for _, eD in objD.items():
                try:
                    for tD in eD["rcsb_entity_source_organism"]:
                        taxIdS.add(tD["ncbi_taxonomy_id"])
                except Exception:
                    pass
                try:
                    for tD in eD["rcsb_entity_host_organism"]:
                        taxIdS.add(tD["ncbi_taxonomy_id"])
                except Exception:
                    pass
            logger.info("Unique taxons %d", len(taxIdS))
            return list(taxIdS)
        except Exception as e:
            logger.exception("Failing with %s", str(e))



##
# File:    ReferenceSequenceAssignmentProviderTests.py
# Author:  J. Westbrook
# Date:    17-Oct-2019
#
# Updates:
#
##
"""
Tests for reference sequence assignment update operations
"""

__docformat__ = "google en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"

import logging
import os
import platform
import resource
import time
import unittest

from rcsb.exdb.seq.ReferenceSequenceAssignmentProvider import ReferenceSequenceAssignmentProvider
from rcsb.utils.config.ConfigUtil import ConfigUtil

logging.basicConfig(level=logging.INFO, format="%(asctime)s [%(levelname)s]-%(module)s.%(funcName)s: %(message)s")
logger = logging.getLogger()

HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(os.path.dirname(HERE)))


class ReferenceSequenceAssignmentProviderTests(unittest.TestCase):
    def __init__(self, methodName="runTest"):
        super(ReferenceSequenceAssignmentProviderTests, self).__init__(methodName)
        self.__verbose = True
        self.__traceMemory = False

    def setUp(self):
        #
        self.__mockTopPath = os.path.join(TOPDIR, "rcsb", "mock-data")
        configPath = os.path.join(TOPDIR, "rcsb", "mock-data", "config", "dbload-setup-example.yml")
        configName = "site_info_configuration"
        self.__cfgOb = ConfigUtil(configPath=configPath, defaultSectionName=configName, mockTopPath=self.__mockTopPath)
        #
        self.__resourceName = "MONGO_DB"
        self.__cachePath = os.path.join(TOPDIR, "CACHE")
        # self.__testEntityCacheKwargs = {"fmt": "json", "indent": 3}
        self.__testEntityCacheKwargs = {"fmt": "pickle"}
        self.__fetchLimitTest = None
        self.__useCache = False
        #
        self.__startTime = time.time()
        logger.debug("Starting %s at %s", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()))

    def tearDown(self):
        unitS = "MB" if platform.system() == "Darwin" else "GB"
        rusageMax = resource.getrusage(resource.RUSAGE_SELF).ru_maxrss
        logger.info("Maximum resident memory size %.4f %s", rusageMax / 10 ** 6, unitS)
        endTime = time.time()
        logger.info("Completed %s at %s (%.4f seconds)", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()), endTime - self.__startTime)

    def testAssignmentProvider(self):
        """Test case - create and read cache reference sequences assignments and related data.

        Some profiling statistics -
         Current memory usage is 0.711864MB; Peak was 4646.476926MB (full cache no limit)
         Current memory usage is 1.080839MB; Peak was 1258.231275MB (163)
         Current memory usage is 0.874476MB; Peak was 1920.689116MB (918)
         Current memory usage is 0.937091MB; Peak was 2086.910197MB (2739)
         Current memory usage is 1.3539 MB; Peak was 2300.5170 MB (10000)
         Current memory usage is 1.3517 MB; Peak was 2714.5467 MB (20K entries)
        """
        try:
            #  -- create cache ---
            rsaP = ReferenceSequenceAssignmentProvider(
                self.__cfgOb,
                databaseName="pdbx_core",
                collectionName="pdbx_core_polymer_entity",
                polymerType="Protein",
                referenceDatabaseName="UniProt",
                provSource="PDB",
                useCache=self.__useCache,
                cachePath=self.__cachePath,
                cacheKwargs=self.__testEntityCacheKwargs,
                fetchLimit=self.__fetchLimitTest,
                siftsAbbreviated="TEST",
            )
            ok = rsaP.testCache()
            self.assertTrue(ok)
            numRef = rsaP.getRefDataCount()
            self.assertGreaterEqual(numRef, 90)
            #
            # ---  Reload from cache ---
            rsaP = ReferenceSequenceAssignmentProvider(
                self.__cfgOb, referenceDatabaseName="UniProt", useCache=True, cachePath=self.__cachePath, cacheKwargs=self.__testEntityCacheKwargs
            )
            ok = rsaP.testCache()
            self.assertTrue(ok)
            numRef = rsaP.getRefDataCount()
            self.assertGreaterEqual(numRef, 90)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()


def referenceSequenceAssignmentProviderSuite():
    suiteSelect = unittest.TestSuite()
    suiteSelect.addTest(ReferenceSequenceAssignmentProviderTests("testAssignmentProvider"))
    return suiteSelect


if __name__ == "__main__":
    mySuite = referenceSequenceAssignmentProviderSuite()
    unittest.TextTestRunner(verbosity=2).run(mySuite)



##
#  File:           GlycanProvider.py
#  Date:           24-May-2021 jdw
#
#  Updated:
#
##
"""
Accessors for glycan mapped annotations.

"""

import logging
import os.path
import time

from rcsb.exdb.branch.GlycanUtils import GlycanUtils
from rcsb.utils.io.MarshalUtil import MarshalUtil
from rcsb.utils.io.StashableBase import StashableBase

logger = logging.getLogger(__name__)


class GlycanProvider(StashableBase):
    """Accessors and generators for entity glycan mapped identifiers.

    dirPath -> CACHE/glycan/
                             branched_entity_glycan_identifier_map.json
                             accession-wurcs-mapping.json
                     stash/entity_glycan_mapped_identifiers.tar.gz

    """

    def __init__(self, **kwargs):
        #
        self.__version = "0.50"
        cachePath = kwargs.get("cachePath", ".")
        useCache = kwargs.get("useCache", True)
        self.__dirName = "glycan"
        self.__dirPath = os.path.join(cachePath, self.__dirName)
        super(GlycanProvider, self).__init__(cachePath, [self.__dirName])
        #
        self.__mU = MarshalUtil(workPath=self.__dirPath)
        self.__glyD = self.__reload(fmt="json", useCache=useCache)
        #

    def testCache(self, minCount=1):
        if minCount == 0:
            return True
        if self.__glyD and minCount and ("identifiers" in self.__glyD) and len(self.__glyD["identifiers"]) >= minCount:
            logger.info("Glycan identifiers (%d)", len(self.__glyD["identifiers"]))
            return True
        return False

    def getIdentifiers(self):
        """Return a dictionary of related identifiers organized by branched entity id.

        Returns:
            (dict): {entityId: {'idType1': ids, 'idType1': ids}, ... }
        """
        try:
            return self.__glyD["identifiers"] if self.__glyD["identifiers"] else {}
        except Exception as e:
            logger.error("Failing with %r", str(e))
        return {}

    def __getMappingFilePath(self, fmt="json"):
        baseFileName = "branched_entity_glycan_identifier_map"
        fExt = ".json" if fmt == "json" else ".pic"
        fp = os.path.join(self.__dirPath, baseFileName + fExt)
        return fp

    def update(self, cfgOb, fmt="json", indent=3):
        """Update branched entity glycan accession mapping cache.

        Args:
            cfgObj (object): ConfigInfo() object instance

        Returns:
            (bool): True for success for False otherwise
        """
        ok = False
        try:
            gU = GlycanUtils(cfgOb, self.__dirPath)
            eaD = gU.updateEntityAccessionMap()
            logger.info("Got branched entity glycan accession map (%d)", len(eaD))
            #
            tS = time.strftime("%Y %m %d %H:%M:%S", time.localtime())
            self.__glyD = {"version": self.__version, "created": tS, "identifiers": eaD}
            #
            mappingFilePath = self.__getMappingFilePath(fmt=fmt)
            kwargs = {"indent": indent} if fmt == "json" else {}
            ok = self.__mU.doExport(mappingFilePath, self.__glyD, fmt=fmt, **kwargs)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return ok

    def reload(self):
        """Reload from the current cache file."""
        ok = False
        try:
            self.__glyD = self.__reload(fmt="json", useCache=True)
            ok = self.__glyD is not None
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return ok

    def __reload(self, fmt="json", useCache=True):
        mappingFilePath = self.__getMappingFilePath(fmt=fmt)
        tS = time.strftime("%Y %m %d %H:%M:%S", time.localtime())
        pcD = {"version": self.__version, "created": tS, "identifiers": {}}

        if useCache and self.__mU.exists(mappingFilePath):
            logger.info("reading cached path %r", mappingFilePath)
            pcD = self.__mU.doImport(mappingFilePath, fmt=fmt)
        return pcD



##
# File: ChemRefExtractor.py
# Date: 2-Jul-2018  jdw
#
# Selected utilities to extract data from chemical component core collections.
#
# Updates:
#  7-Jan-2019  jdw moved from ChemRefEtlWorker.
#  3-Sep-2019  jdw moved again to module rcsb.exdb.chemref
#
##
__docformat__ = "google en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"

import logging

from rcsb.exdb.utils.ObjectExtractor import ObjectExtractor

logger = logging.getLogger(__name__)


class ChemRefExtractor(object):
    """Selected utilities to extract data from chemical component core collections."""

    def __init__(self, cfgOb):
        self.__cfgOb = cfgOb
        self.__resourceName = "MONGO_DB"
        #

    def getChemCompAccessionMapping(self, referenceResourceName):
        """Get the accession code mapping between chemical component identifiers and identifier(s) for the
            input external reference resource.

        Args:
            referenceResourceName (str):  resource name (e.g. DrugBank, ChEMBL, CCDC)

        Returns:
            dict: {referenceResourceId: chem_comp/bird_id, referenceResourceId: chem_comp/bird_id, ...  }

        """
        idD = {}
        try:
            databaseName = "bird_chem_comp_core"
            collectionName = "bird_chem_comp_core"
            selectD = {"rcsb_chem_comp_related.resource_name": referenceResourceName}
            selectionList = ["rcsb_id", "rcsb_chem_comp_related"]
            logger.info("Searching %s %s with selection query %r", databaseName, collectionName, selectD)
            obEx = ObjectExtractor(
                self.__cfgOb,
                databaseName=databaseName,
                collectionName=collectionName,
                keyAttribute="rcsb_id",
                uniqueAttributes=["rcsb_id"],
                selectionQuery=selectD,
                selectionList=selectionList,
                stripObjectId=True,
            )
            logger.info("Reference data object count %d", obEx.getCount())
            objD = obEx.getObjects()
            for _, doc in objD.items():
                dL = doc["rcsb_chem_comp_related"] if "rcsb_chem_comp_related" in doc else []
                for dD in dL:
                    if dD["resource_name"] == referenceResourceName and "resource_accession_code" in dD:
                        idD.setdefault(dD["resource_accession_code"], []).append(dD["comp_id"])
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return idD



##
# File: PubChemDataCacheProvider.py
# Date: 2-Apr-2020  jdw
#
# Utilities to cache chemical reference data and mappings for PubChem
#
# Updates:
#  9-May-2020 jdw separate cache behavior with separate option rebuildChemIndices=True/False
# 16-Jul-2020 jdw separate index and reference data management.
# 23-Jul-2021 jdw Make PubChemDataCacheProvider a subclass of StashableBase()
#
##
__docformat__ = "google en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"

import logging
import os
import time

from rcsb.exdb.utils.ObjectExtractor import ObjectExtractor
from rcsb.exdb.utils.ObjectUpdater import ObjectUpdater
from rcsb.utils.chemref.PubChemUtils import PubChemUtils, ChemicalIdentifier
from rcsb.utils.io.IoUtil import getObjSize
from rcsb.utils.io.MarshalUtil import MarshalUtil
from rcsb.utils.io.StashableBase import StashableBase
from rcsb.utils.io.TimeUtil import TimeUtil
from rcsb.utils.multiproc.MultiProcUtil import MultiProcUtil


logger = logging.getLogger(__name__)


class PubChemDataUpdateWorker(object):
    """A skeleton worker class that implements the interface expected by the multiprocessing module
    for fetching PubChem chemical reference data --
    """

    def __init__(self, cfgOb, **kwargs):
        self.__cfgOb = cfgOb
        #
        _ = kwargs
        self.__databaseName = "pubchem_exdb"
        self.__refDataCollectionName = "reference_entry"
        self.__createCollections(self.__databaseName, self.__refDataCollectionName, indexAttributeNames=["rcsb_id", "rcsb_last_update"])
        self.__pcU = PubChemUtils()

    def updateList(self, dataList, procName, optionsD, workingDir):
        """Update the input list of reference data identifiers (ChemicalIdentifier()) and return
        matching diagnostics and reference feature data.

        """
        _ = workingDir
        chunkSize = optionsD.get("chunkSize", 50)
        # Path to store raw request data -
        exportPath = optionsD.get("exportPath", None)
        #
        successList = []
        retList1 = []
        retList2 = []
        diagList = []
        emptyList = []
        # -
        try:
            tU = TimeUtil()
            pcidList = dataList
            numChunks = len(list(self.__chunker(pcidList, chunkSize)))
            logger.info("%s search starting for %d reference definitions (in chunks of length %d)", procName, len(pcidList), chunkSize)
            for ii, pcidChunk in enumerate(self.__chunker(pcidList, chunkSize), 1):
                logger.info("%s starting chunk for %d of %d", procName, ii, numChunks)
                tDL = []
                timeS = tU.getDateTimeObj(tU.getTimestamp())
                for pcid in pcidChunk:
                    #
                    chemId = ChemicalIdentifier(idCode=pcid, identifierType="cid", identifier=pcid, identifierSource="ccd-match")
                    #
                    stA = time.time()
                    ok, refDL = self.__pcU.assemble(chemId, exportPath=exportPath)
                    #
                    if not ok:
                        etA = time.time()
                        logger.info("Failing %s search source %s for %s (%.4f secs)", chemId.identifierType, chemId.identifierSource, chemId.idCode, etA - stA)

                    #
                    if ok and refDL:
                        successList.append(pcid)
                        for tD in refDL:
                            tD.update({"rcsb_id": tD["cid"], "rcsb_last_update": timeS})
                            tDL.append(tD)
                    else:
                        logger.info("No match result for any form of %s", pcid)
                # --
                startTimeL = time.time()
                logger.info("Saving chunk %d (len=%d)", ii, len(pcidChunk))
                self.__updateObjectStore(self.__databaseName, self.__refDataCollectionName, tDL)
                endTimeL = time.time()
                logger.info("Saved chunk %d (len=%d) in %.3f secs", ii, len(pcidChunk), endTimeL - startTimeL)
        except Exception as e:
            logger.exception("Failing %s for %d data items %s", procName, len(dataList), str(e))
        logger.info("%s dataList length %d success length %d rst1 %d rst2 %d", procName, len(dataList), len(successList), len(retList1), len(retList2))
        #
        return successList, emptyList, emptyList, diagList

    def __updateObjectStore(self, databaseName, collectionName, objDL):
        updateDL = []
        for objD in objDL:
            try:
                selectD = {"rcsb_id": objD["rcsb_id"]}
                updateDL.append({"selectD": selectD, "updateD": objD})
            except Exception as e:
                logger.exception("Failing with %s", str(e))
        obUpd = ObjectUpdater(self.__cfgOb)
        numUpd = obUpd.update(databaseName, collectionName, updateDL)
        logger.info("Updated reference count is %d", numUpd)

    def __createCollections(self, databaseName, collectionName, indexAttributeNames=None):
        obUpd = ObjectUpdater(self.__cfgOb)
        ok = obUpd.createCollection(databaseName, collectionName, indexAttributeNames=indexAttributeNames, checkExists=True, bsonSchema=None)
        return ok

    def __chunker(self, iList, chunkSize):
        chunkSize = max(1, chunkSize)
        return (iList[i : i + chunkSize] for i in range(0, len(iList), chunkSize))


class PubChemDataCacheProvider(StashableBase):
    """Utilities to cache chemical reference data extracted from PubChem compound data"""

    def __init__(self, cfgOb, cachePath):
        dirName = "PubChem-data"
        super(PubChemDataCacheProvider, self).__init__(cachePath, [dirName])
        self.__cfgOb = cfgOb
        self.__dirPath = os.path.join(cachePath, dirName)
        #
        self.__databaseName = "pubchem_exdb"
        self.__refDataCollectionName = "reference_entry"
        #
        self.__refD = None

    def getRefData(self, expireDays=0):
        if not self.__refD:
            selectD = {}
            if expireDays > 0:
                tU = TimeUtil()
                tS = tU.getTimestamp(useUtc=True, before={"days": expireDays})
                selectD.update({"rcsb_latest_update": {"$lt": tU.getDateTimeObj(tS)}})
            self.__refD = self.__getReferenceData(self.__databaseName, self.__refDataCollectionName, selectD=selectD)
            #
        return self.__refD

    def getRefIdCodes(self, expireDays=0):
        selectD = {}
        selectionList = ["rcsb_id"]
        if expireDays > 0:
            tU = TimeUtil()
            tS = tU.getTimestamp(useUtc=True, before={"days": expireDays})
            selectD.update({"rcsb_latest_update": {"$lt": tU.getDateTimeObj(tS)}})
        refIds = self.__getReferenceData(self.__databaseName, self.__refDataCollectionName, selectD=selectD, selectionList=selectionList)
        #
        return list(refIds.keys()) if refIds else []

    def getRefDataCount(self):
        return len(self.__refD) if self.__refD else 0

    def testCache(self, minCount=None, logSizes=False):
        okC = bool(self.__refD)
        if not okC:
            return okC
        logger.info("Reference data cache lengths: refD %d", len(self.__refD))
        if minCount and len(self.__refD) < minCount:
            return False
        #
        if logSizes:
            logger.info("refD %.2f", getObjSize(self.__refD) / 1000000.0)
        return True

    def __getdumpFilePath(self, fmt="json"):
        stashBaseFileName = "pubchem_match_data_object_list"
        fExt = ".json" if fmt == "json" else ".pic"
        fp = os.path.join(self.__dirPath, stashBaseFileName + fExt)
        return fp

    def dump(self, fmt="json"):
        """Dump PubChem reference data from the object store.

        Args:
            fmt (str, optional): backup file format. Defaults to "json".

        Returns:
            (bool): True for success or False otherwise
        """
        ok = False
        try:
            self.getRefData()
            if fmt in ["json", "pickle"]:
                kwargs = {}
                fp = self.__getdumpFilePath(fmt=fmt)
                logger.info("Saving object store to %s", fp)
                mU = MarshalUtil(workPath=self.__dirPath)
                if fmt in ["json"]:
                    kwargs = {"indent": 3}
                ok = mU.doExport(fp, self.__refD, fmt=fmt, **kwargs)
        except Exception as e:
            logger.exception("Failing for %r with %s", self.__dirPath, str(e))
        return ok

    def reloadDump(self, fmt="json"):
        """Load PubChem reference data store from saved dump.

        Args:
            fmt (str, optional): format of the backup file (pickle or json). Defaults to "json".

        Returns:
            (int): number of objects restored.
        """
        numUpd = 0
        try:
            # Read from disk backup and update object store -
            if fmt in ["json", "pickle"]:
                fp = self.__getdumpFilePath(fmt=fmt)
                logger.info("Restoring object store from %s", fp)
                mU = MarshalUtil(workPath=self.__dirPath)
                refD = mU.doImport(fp, fmt=fmt)
                numUpd = self.__reloadDump(refD, self.__databaseName, self.__refDataCollectionName, indexAttributeNames=["rcsb_id", "rcsb_last_update"])
        except Exception as e:
            logger.exception("Failing for %r with %s", self.__dirPath, str(e))
        # --
        return numUpd

    def updateMissing(self, idList, exportPath=None, numProc=1, chunkSize=5):
        """Fetch and load reference data for any missing PubChem ID codes in the input list.

        Args:
            idList (list): PubChem ID codes
            numProc (int, optional): number of processor to use. Defaults to 1.
            chunkSize (int, optional): chunk size between data store updates. Defaults to 5.
            exportPath (str, optional): store raw fetched data in this path. Defaults to None.

        Returns:
            (bool, list): status flag, list of failed identifiers
        """
        curIdList = self.getRefIdCodes()
        missS = set(idList) - set(curIdList)
        if missS:
            logger.info("Loading (%d) missing identifiers", len(missS))
            ok, failList = self.load(list(missS), numProc=numProc, chunkSize=chunkSize, exportPath=exportPath)
        else:
            logger.info("No missing identifier - nothing to load")
            ok = True
            failList = []

        return ok, failList

    def load(self, idList, exportPath=None, numProc=1, chunkSize=5):
        """Fetch and load reference data for the input list of PubChem compound codes.

        Args:
            idList (list): PubChem ID codes
            exportPath (str, optional): store raw fetched data in this path. Defaults to None.
            numProc (int, optional): number of processor to use. Defaults to 1.
            chunkSize (int, optional): chunk size between data store updates. Defaults to 5.


        Returns:
            (bool, list): status flag, list of failed identifiers

        """
        logger.info("Length starting list is %d", len(idList))
        optD = {"chunkSize": chunkSize, "exportPath": exportPath}
        rWorker = PubChemDataUpdateWorker(self.__cfgOb)
        if numProc > 1:
            mpu = MultiProcUtil(verbose=True)
            mpu.setOptions(optD)
            mpu.set(workerObj=rWorker, workerMethod="updateList")
            ok, failList, resultList, _ = mpu.runMulti(dataList=idList, numProc=numProc, numResults=2, chunkSize=chunkSize)
            logger.info("Multi-proc %r failures %r result lengths %r %r", ok, len(failList), len(resultList[0]), len(resultList[1]))
        else:
            successList, _, _, _ = rWorker.updateList(idList, "SingleProc", optD, None)
            failList = list(set(idList) - set(successList))
            ok = len(failList) == 0
            logger.info("Single-proc status %r failures %r", ok, len(failList))
        #
        return ok, failList

    def getRelatedMapping(self, pcidList):
        """Assemble related identifiers (xrefs) for the input PubChem compound Id list.

        Args:
            pcidList (list): PubChem compound ID list

        Returns:
            dict :{<pcid>: {'relatedId1': ... 'relatedId2': ... }, ...}

        """
        #
        retD = {}
        logger.info("Get XREFs for PubChem compound ID list (%d)", len(pcidList))
        #
        try:
            xrefD = self.__getReferenceData(self.__databaseName, self.__refDataCollectionName, selectD=None, selectionList=["rcsb_id", "cid", "data.xrefs"])
            for pcid in pcidList:
                try:
                    xD = xrefD[pcid]["data"]["xrefs"]
                    if isinstance(xD, list):
                        xD = {}
                except Exception:
                    xD = {}
                #
                mD = {}
                logger.debug("%s (%s) xrefs %r", pcid, xrefD[pcid]["cid"], xD)
                for rNm, rIdL in xD.items():
                    mD[rNm] = rIdL
                retD[pcid] = mD
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        #
        return retD

    #
    def __getReferenceData(self, databaseName, collectionName, selectD=None, selectionList=None):
        logger.info("Searching %s %s with selection query %r", databaseName, collectionName, selectD)
        obEx = ObjectExtractor(
            self.__cfgOb,
            databaseName=databaseName,
            collectionName=collectionName,
            keyAttribute="rcsb_id",
            uniqueAttributes=["rcsb_id"],
            selectionQuery=selectD,
            selectionList=selectionList,
            stripObjectId=True,
        )
        docCount = obEx.getCount()
        logger.info("Reference data object count %d", docCount)
        objD = obEx.getObjects()
        return objD

    def __reloadDump(self, objD, databaseName, collectionName, indexAttributeNames=None):
        """Internal method to restore the input database/collection using the input data object.

        Args:
            objD (obj): Target reference or index data object
            databaseName (str): target database name
            collectionName (str): target collection name
            indexAttributeNames (list, optional): Primary index attributes. Defaults to None.

        Returns:
            int: inserted or updated object count
        """
        try:
            numUpd = 0
            numTotal = 0
            updateDL = []
            for entityKey, obj in objD.items():
                # if "_id" in obj:
                #    obj.pop("_id")
                selectD = {"rcsb_id": entityKey}
                updateDL.append({"selectD": selectD, "updateD": obj})
            #
            obUpd = ObjectUpdater(self.__cfgOb)
            ok = obUpd.createCollection(databaseName, collectionName, indexAttributeNames=indexAttributeNames, checkExists=True, bsonSchema=None)
            if ok:
                numUpd = obUpd.update(databaseName, collectionName, updateDL)
                logger.debug("Updated object count is %d", numUpd)
            else:
                logger.error("Create %s %s failed", databaseName, collectionName)
            numTotal = obUpd.count(databaseName, collectionName)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        #
        return numTotal



##
# File: ChemRefEtlWorker.py
# Date: 2-Jul-2018  jdw
#
# ETL utilities for processing chemical reference data and related data integration.
#
# Updates:
#  9-Dec-2018  jdw add validation methods
#  3-Sep-2019  jdw move to rcsb.exdb.chemref
#
##
__docformat__ = "google en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"

import logging

from rcsb.db.mongo.DocumentLoader import DocumentLoader
from rcsb.db.processors.DataExchangeStatus import DataExchangeStatus
from rcsb.db.utils.SchemaProvider import SchemaProvider
from rcsb.exdb.chemref.ChemRefExtractor import ChemRefExtractor
from rcsb.utils.chemref.DrugBankProvider import DrugBankProvider


logger = logging.getLogger(__name__)


class ChemRefEtlWorker(object):
    """Prepare and load chemical reference data collections."""

    def __init__(self, cfgOb, cachePath, useCache=True, numProc=2, chunkSize=10, readBackCheck=False, documentLimit=None, verbose=False):
        self.__cfgOb = cfgOb
        self.__cachePath = cachePath
        self.__useCache = useCache
        self.__readBackCheck = readBackCheck
        self.__numProc = numProc
        self.__chunkSize = chunkSize
        self.__documentLimit = documentLimit
        #
        self.__resourceName = "MONGO_DB"
        self.__verbose = verbose
        self.__statusList = []
        self.__schP = SchemaProvider(self.__cfgOb, self.__cachePath, useCache=self.__useCache)
        #

    def __updateStatus(self, updateId, databaseName, collectionName, status, startTimestamp):
        try:
            sFlag = "Y" if status else "N"
            desp = DataExchangeStatus()
            desp.setStartTime(tS=startTimestamp)
            desp.setObject(databaseName, collectionName)
            desp.setStatus(updateId=updateId, successFlag=sFlag)
            desp.setEndTime()
            self.__statusList.append(desp.getStatus())
            return True
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return False

    def load(self, updateId, extResource, loadType="full"):
        """Load chemical reference integrated data for the input external resource-"""
        try:
            self.__statusList = []
            desp = DataExchangeStatus()
            statusStartTimestamp = desp.setStartTime()
            #
            if extResource == "DrugBank":
                databaseName = "drugbank_core"
                configName = self.__cfgOb.getDefaultSectionName()
                user = self.__cfgOb.get("_DRUGBANK_AUTH_USERNAME", sectionName=configName)
                pw = self.__cfgOb.get("_DRUGBANK_AUTH_PASSWORD", sectionName=configName)
                #
                dbP = DrugBankProvider(cachePath=self.__cachePath, useCache=self.__useCache, username=user, password=pw)
                #
                crExt = ChemRefExtractor(self.__cfgOb)
                idD = crExt.getChemCompAccessionMapping(extResource)
                dList = dbP.getDocuments(mapD=idD)
                #
                logger.info("Resource %r extracted mapped document length %d", extResource, len(dList))
                logger.debug("Objects %r", dList[:2])
                sD, _, collectionList, _ = self.__schP.getSchemaInfo(databaseName)
                collectionName = collectionList[0] if collectionList else "unassigned"
                indexL = sD.getDocumentIndex(collectionName, "primary")
                logger.info("Database %r collection %r index attributes %r", databaseName, collectionName, indexL)
                #
                collectionVersion = sD.getCollectionVersion(collectionName)
                addValues = {"_schema_version": collectionVersion}
                #
                addValues = {}
            #
            dl = DocumentLoader(
                self.__cfgOb,
                self.__cachePath,
                self.__resourceName,
                numProc=self.__numProc,
                chunkSize=self.__chunkSize,
                documentLimit=self.__documentLimit,
                verbose=self.__verbose,
                readBackCheck=self.__readBackCheck,
            )
            #
            ok = dl.load(databaseName, collectionName, loadType=loadType, documentList=dList, indexAttributeList=indexL, keyNames=None, addValues=addValues)
            self.__updateStatus(updateId, databaseName, collectionName, ok, statusStartTimestamp)

            return True
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return False

    def getLoadStatus(self):
        return self.__statusList



##
#  File:           GlycanUtils.py
#  Date:           24-May-2021 jdw
#
#  Updated:
##
"""
Utilities for fetching and mapping glycan accessions.
"""

import logging
import os.path

from rcsb.exdb.branch.BranchedEntityExtractor import BranchedEntityExtractor
from rcsb.utils.io.MarshalUtil import MarshalUtil
from rcsb.utils.io.UrlRequestUtil import UrlRequestUtil

logger = logging.getLogger(__name__)


class GlycanUtils:
    """Utilities for fetching and mapping glycan annotations."""

    def __init__(self, cfgOb, dirPath):
        self.__cfgOb = cfgOb
        self.__dirPath = dirPath
        self.__mU = MarshalUtil(workPath=self.__dirPath)
        #

    def __getRawGlycanDetailsPath(self):
        return os.path.join(self.__dirPath, "pdb-raw-branched-entity-details.json")

    def getBranchedEntityDetails(self):
        """For branched entities, get BIRD mapping and WURCS details"""
        ok = False
        try:
            bEx = BranchedEntityExtractor(self.__cfgOb)
            branchedEntityD = bEx.getBranchedDetails()
            logger.info("Branched entity descriptor details count %d", len(branchedEntityD))
            detailsPath = self.__getRawGlycanDetailsPath()
            ok = bEx.exportBranchedEntityDetails(detailsPath, fmt="json")
            logger.info("Store raw branched entity data (%r) %s", ok, detailsPath)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        #
        return branchedEntityD

    def __getGlycanAccessionMapPath(self):
        return os.path.join(self.__dirPath, "accession-wurcs-mapping.json")

    def fetchGlycanAccessionMap(self):
        mapD = {}
        accessionMapPath = self.__getGlycanAccessionMapPath()
        if self.__mU.exists(accessionMapPath):
            mapD = self.__mU.doImport(accessionMapPath, fmt="json")
        return mapD

    def storeGlycanAccessionMap(self, mapD):
        accessionMapPath = self.__getGlycanAccessionMapPath()
        ok = self.__mU.doExport(accessionMapPath, mapD, fmt="json", indent=3)
        return ok

    def updateEntityAccessionMap(self):
        """Update entity to glycan accession mapping

        Returns:
            dict: {entityId: {'glyTouCanId':... , 'prdId': ..., }, ... }
        """
        entityAccessionMapD = {}
        wurcsTupL = []
        uniqueWurcsD = {}
        accessionMapD = self.fetchGlycanAccessionMap()
        branchedEntityD = self.getBranchedEntityDetails()
        for entityId, iD in branchedEntityD.items():
            if iD["wurcs"] and iD["wurcs"] not in accessionMapD and iD["wurcs"] not in uniqueWurcsD:
                wurcsTupL.append((entityId, iD["wurcs"]))
                uniqueWurcsD.setdefault(iD["wurcs"], []).append(entityId)
        if wurcsTupL:
            tMap = self.getAccessionMapping(wurcsTupL)
            accessionMapD.update(tMap)
            self.storeGlycanAccessionMap(accessionMapD)
        #

        for entityId, iD in branchedEntityD.items():
            if iD["wurcs"] in accessionMapD:
                prdId = iD["prdId"] if iD["wurcs"] else None
                entityAccessionMapD[entityId] = {"glyTouCanId": accessionMapD[iD["wurcs"]][0], "prdId": prdId}
        return entityAccessionMapD

    def getAccessionMapping(self, wurcsTupL):
        """Fetch GlyTouCan accessions for the input WURCS desriptor list"""
        accessionMapD = {}
        logger.info("Fetching (%d) WURCS descriptors", len(wurcsTupL))
        baseUrl = "https://api.glycosmos.org"
        endPoint = "glytoucan/sparql/wurcs2gtcids"
        numDescriptors = len(wurcsTupL)
        for ii, (entityId, wurcs) in enumerate(wurcsTupL, 1):
            try:
                pD = {}
                pD["wurcs"] = wurcs
                uR = UrlRequestUtil()
                rDL, retCode = uR.post(baseUrl, endPoint, pD, returnContentType="JSON")
                logger.debug(" %r wurcs fetch result (%r) %r", entityId, retCode, rDL)
                if rDL:
                    for rD in rDL:
                        if "id" in rD:
                            accessionMapD.setdefault(wurcs, []).append(rD["id"])
                        else:
                            logger.info("%r fetch fails (%r) (%r) %r", entityId, retCode, wurcs, rDL)
                if ii % 5 == 0:
                    logger.info("Fetched %d/%d", ii, numDescriptors)
            except Exception as e:
                logger.exception("Failing for (%r) wurcs (%r) with %s", entityId, wurcs, str(e))
        return accessionMapD



##
# File:    ChemRefLoaderTests.py
# Author:  J. Westbrook
# Date:    18-Jun-2021
#
# Updates:
#
##
"""
Tests for loading chemical reference data and identifer mapping information.

"""

__docformat__ = "google en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"


import logging
import os
import platform
import resource
import time
import unittest

from rcsb.exdb.chemref.ChemRefExtractor import ChemRefExtractor
from rcsb.exdb.chemref.ChemRefEtlWorker import ChemRefEtlWorker
from rcsb.utils.config.ConfigUtil import ConfigUtil
from rcsb.utils.io.MarshalUtil import MarshalUtil

logging.basicConfig(level=logging.INFO, format="%(asctime)s [%(levelname)s]-%(module)s.%(funcName)s: %(message)s")
logger = logging.getLogger()

HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(os.path.dirname(HERE)))


class ChemRefLoaderTests(unittest.TestCase):
    def __init__(self, methodName="runTest"):
        super(ChemRefLoaderTests, self).__init__(methodName)
        self.__verbose = True

    def setUp(self):
        #
        #
        self.__mockTopPath = os.path.join(TOPDIR, "rcsb", "mock-data")
        configPath = os.path.join(TOPDIR, "rcsb", "mock-data", "config", "dbload-setup-example.yml")
        configName = "site_info_configuration"
        self.__cfgOb = ConfigUtil(configPath=configPath, defaultSectionName=configName, mockTopPath=self.__mockTopPath)
        self.__cachePath = os.path.join(TOPDIR, "CACHE")
        #
        # sample data set
        self.__updateId = "2021_10"
        #
        self.__startTime = time.time()
        logger.debug("Starting %s at %s", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()))

    def tearDown(self):
        unitS = "MB" if platform.system() == "Darwin" else "GB"
        rusageMax = resource.getrusage(resource.RUSAGE_SELF).ru_maxrss
        logger.info("Maximum resident memory size %.4f %s", rusageMax / 10 ** 6, unitS)
        endTime = time.time()
        logger.info("Completed %s at %s (%.4f seconds)", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()), endTime - self.__startTime)

    def testLoadIntegratedDrugBankData(self):
        """Test case - load integrated DrugBank chemical reference data -"""
        try:
            crw = ChemRefEtlWorker(self.__cfgOb, self.__cachePath)
            crExt = ChemRefExtractor(self.__cfgOb)

            idD = crExt.getChemCompAccessionMapping(referenceResourceName="DrugBank")
            logger.info("Mapping dictionary %r", len(idD))
            #
            ok = crw.load(self.__updateId, extResource="DrugBank", loadType="full")
            #
            self.assertTrue(ok)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def testDrugBankDataMapping(self):
        """Test case - get DrugBank mapping -"""
        try:
            crExt = ChemRefExtractor(self.__cfgOb)
            idD = crExt.getChemCompAccessionMapping(referenceResourceName="DrugBank")
            logger.info("Mapping dictionary %r", len(idD))
            #
            mU = MarshalUtil()
            fp = os.path.join(HERE, "test-output", "drugbank-mapping.json")
            mU.doExport(fp, idD, fmt="json", indent=3)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()


def chemRefLoadSuite():
    suiteSelect = unittest.TestSuite()
    suiteSelect.addTest(ChemRefLoaderTests("testLoadIntegratedDrugBankData"))
    return suiteSelect


if __name__ == "__main__":
    #
    mySuite = chemRefLoadSuite()
    unittest.TextTestRunner(verbosity=2).run(mySuite)



##
# File:    UniProtCoreEtlWorkerTests.py
# Author:  J. Westbrook
# Date:    9-Dec-2018
#
# Updates:
#
##
"""
Tests for loading UniProt core collection

"""

__docformat__ = "google en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"


import logging
import os
import platform
import resource
import time
import unittest

from rcsb.exdb.seq.UniProtCoreEtlWorker import UniProtCoreEtlWorker
from rcsb.utils.config.ConfigUtil import ConfigUtil

logging.basicConfig(level=logging.INFO, format="%(asctime)s [%(levelname)s]-%(module)s.%(funcName)s: %(message)s")
logger = logging.getLogger()

HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(os.path.dirname(HERE)))


class UniProtCoreEtlWorkerTests(unittest.TestCase):
    def __init__(self, methodName="runTest"):
        super(UniProtCoreEtlWorkerTests, self).__init__(methodName)
        self.__verbose = True

    def setUp(self):
        #
        #
        self.__mockTopPath = os.path.join(TOPDIR, "rcsb", "mock-data")
        configPath = os.path.join(TOPDIR, "rcsb", "mock-data", "config", "dbload-setup-example.yml")
        configName = "site_info_configuration"
        self.__cfgOb = ConfigUtil(configPath=configPath, defaultSectionName=configName, mockTopPath=self.__mockTopPath)
        self.__cachePath = os.path.join(TOPDIR, "CACHE")
        #
        # sample data set
        self.__updateId = "2018_23"
        #
        self.__startTime = time.time()
        logger.debug("Starting %s at %s", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()))

    def tearDown(self):
        unitS = "MB" if platform.system() == "Darwin" else "GB"
        rusageMax = resource.getrusage(resource.RUSAGE_SELF).ru_maxrss
        logger.info("Maximum resident memory size %.4f %s", rusageMax / 10 ** 6, unitS)
        endTime = time.time()
        logger.info("Completed %s at %s (%.4f seconds)", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()), endTime - self.__startTime)

    @unittest.skip("Disable test - deprecated")
    def testLoadUniProtCore(self):
        """Test case - load UniProt core collection reference data -"""
        try:
            uw = UniProtCoreEtlWorker(self.__cfgOb, self.__cachePath)
            ok = uw.load(self.__updateId, extResource="UniProt", loadType="full")
            #
            self.assertTrue(ok)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    @unittest.skip("Disable test - deprecated")
    def testValidateUniProtCore(self):
        """Test case - validate UniProt core collection reference data -"""
        try:
            uw = UniProtCoreEtlWorker(self.__cfgOb, self.__cachePath, doValidate=True)
            ok = uw.load(self.__updateId, extResource="UniProt", loadType="full")
            #
            self.assertTrue(ok)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()


def uniProtCoreEtlWorkerSuite():
    suiteSelect = unittest.TestSuite()
    suiteSelect.addTest(UniProtCoreEtlWorkerTests("testLoadUniProtCore"))
    suiteSelect.addTest(UniProtCoreEtlWorkerTests("testValidateUniProtCore"))
    return suiteSelect


if __name__ == "__main__":
    #
    mySuite = uniProtCoreEtlWorkerSuite()
    unittest.TextTestRunner(verbosity=2).run(mySuite)



##
# File: PubChemEtlWorkflow.py
# Date: 28-Jul-2020  jdw
#
#  Workflow wrapper  --  PubChem ETL utilities
#
#  Updates:
#
##
__docformat__ = "google en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"

import logging
import os
import time


from rcsb.exdb.chemref.PubChemEtlWrapper import PubChemEtlWrapper
from rcsb.utils.config.ConfigUtil import ConfigUtil

logger = logging.getLogger(__name__)


class PubChemEtlWorkflow(object):
    def __init__(self, **kwargs):
        """Workflow wrapper  --  PubChem ETL utilities

        Args:
            configPath (str, optional): path to configuration file (default: exdb-config-example.yml)
            configName (str, optional): configuration section name (default: site_info_configuration)
            cachePath (str, optional):  path to cache directory (default: '.')
            stashRemotePrefix (str, optional): file name prefix (channel) applied to remote stash file artifacts (default: None)
        """
        configPath = kwargs.get("configPath", "exdb-config-example.yml")
        self.__configName = kwargs.get("configName", "site_info_configuration")
        mockTopPath = kwargs.get("mockTopPath", None)
        self.__cfgOb = ConfigUtil(configPath=configPath, defaultSectionName=self.__configName, mockTopPath=mockTopPath)
        #
        self.__cachePath = kwargs.get("cachePath", ".")
        self.__cachePath = os.path.abspath(self.__cachePath)
        self.__stashRemotePrefix = kwargs.get("stashRemotePrefix", None)
        #
        self.__debugFlag = kwargs.get("debugFlag", False)
        if self.__debugFlag:
            logger.setLevel(logging.DEBUG)
        #

    def dump(self):
        """Dump the current object store of PubChem correspondences and data."""
        ok1 = ok2 = ok3 = ok4 = False
        try:
            #  -- Update local chemical indices and  create PubChem mapping index ---
            pcewP = PubChemEtlWrapper(self.__cfgOb, self.__cachePath, stashRemotePrefix=self.__stashRemotePrefix)
            sTime = time.time()
            logger.info("Dumping index data")
            ok1 = pcewP.dump(contentType="index")
            ok2 = pcewP.toStash(contentType="index")
            eTime = time.time()
            logger.info("Dumping index data done in (%.4f seconds)", eTime - sTime)

            sTime = time.time()
            logger.info("Dumping reference data")
            ok3 = pcewP.dump(contentType="data")
            ok4 = pcewP.toStash(contentType="data")
            eTime = time.time()
            logger.info("Dumping data done in (%.4f seconds)", eTime - sTime)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return ok1 and ok2 and ok3 and ok4

    def stash(self):
        """Stash the current cache files containing PubChem correspondences and data."""
        ok1 = ok2 = False
        try:
            #  -- Update local chemical indices and  create PubChem mapping index ---
            pcewP = PubChemEtlWrapper(self.__cfgOb, self.__cachePath, stashRemotePrefix=self.__stashRemotePrefix)
            sTime = time.time()
            ok1 = pcewP.toStash(contentType="index")
            eTime = time.time()
            logger.info("Stashing index data done in (%.4f seconds)", eTime - sTime)

            sTime = time.time()
            logger.info("Stashing reference data")
            ok2 = pcewP.toStash(contentType="data")
            eTime = time.time()
            logger.info("Stashing data done in (%.4f seconds)", eTime - sTime)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return ok1 and ok2

    def restore(self):
        """Restore the current object store of PubChem correspondences and data from stashed data sets."""
        ok1 = ok2 = False
        numObjData = numObjIndex = 0
        try:
            #  -- Update local chemical indices and  create PubChem mapping index ---
            pcewP = PubChemEtlWrapper(self.__cfgOb, self.__cachePath, stashRemotePrefix=self.__stashRemotePrefix)
            sTime = time.time()
            logger.info("Restoring stashed index data")
            ok1 = pcewP.fromStash(contentType="index")
            numObjIndex = pcewP.reloadDump(contentType="index")
            eTime = time.time()
            logger.info("Restoring index data done in (%.4f seconds)", eTime - sTime)

            sTime = time.time()
            logger.info("Restoring reference data")
            ok2 = pcewP.fromStash(contentType="data")
            numObjData = pcewP.reloadDump(contentType="data")
            eTime = time.time()
            logger.info("Restoring data done in (%.4f seconds)", eTime - sTime)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return ok1 and ok2 and numObjData > 1 and numObjIndex > 1

    def updateMatchedIndex(self, **kwargs):
        """Update chemical indices and correspondence matches.

        Args:
            ccUrlTarget (str, optional): target url for chemical component dictionary resource file (default: None=all public)
            birdUrlTarget (str, optional): target url for bird dictionary resource file (cc format) (default: None=all public)
            ccFileNamePrefix (str, optional): index file prefix (default: full)
            rebuildChemIndices (bool, optional): rebuild indices from source (default: False)
            exportPath(str, optional): path to export raw PubChem search results  (default: None)
            numProc(int):  number processors to include in multiprocessing mode (default: 12)

        Returns:
            (bool): True for success or False otherwise

        """
        try:
            ok1 = ok2 = ok3 = False
            #  -- Update local chemical indices and  create PubChem mapping index ---
            ccUrlTarget = kwargs.get("ccUrlTarget", None)
            birdUrlTarget = kwargs.get("birdUrlTarget", None)
            ccFileNamePrefix = kwargs.get("ccFileNamePrefix", "cc-full")
            numProc = kwargs.get("numProc", 12)
            rebuildChemIndices = kwargs.get("rebuildChemIndices", True)
            exportPath = kwargs.get("exportPath", None)
            #
            pcewP = PubChemEtlWrapper(self.__cfgOb, self.__cachePath, stashRemotePrefix=self.__stashRemotePrefix)
            ok1 = pcewP.updateIndex(
                ccUrlTarget=ccUrlTarget,
                birdUrlTarget=birdUrlTarget,
                ccFileNamePrefix=ccFileNamePrefix,
                exportPath=exportPath,
                rebuildChemIndices=rebuildChemIndices,
                numProc=numProc,
            )
            ok2 = pcewP.dump(contentType="index")
            ok3 = pcewP.toStash(contentType="index")
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        #
        return ok1 and ok2 and ok3

    def updateMatchedData(self):
        """Update PubChem annotation data for matched correspondences.  Generate and stash
        related identifiers for corresponding components and BIRD chemical definitions.
        """
        try:
            ok1 = ok2 = ok3 = ok4 = ok5 = ok6 = False
            #  --
            pcewP = PubChemEtlWrapper(self.__cfgOb, self.__cachePath, stashRemotePrefix=self.__stashRemotePrefix)
            ok1 = pcewP.updateMatchedData()
            ok2 = pcewP.dump(contentType="data")
            ok3 = pcewP.toStash(contentType="data")
            #
            ok4 = pcewP.updateIdentifiers()
            ok5 = pcewP.dump(contentType="identifiers")
            ok6 = pcewP.toStash(contentType="identifiers")
            #
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        #
        return ok1 and ok2 and ok3 and ok4 and ok5 and ok6



##
# File: LigandNeighborMappingExtractor.py
# Date: 28-Jun-2021  jdw
#
# Utilities to extract ligand neighbor mapping details from the exchange collections.
#
# Updates:
#
##
__docformat__ = "google en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"

import logging

from rcsb.exdb.utils.ObjectExtractor import ObjectExtractor

logger = logging.getLogger(__name__)


class LigandNeighborMappingExtractor(object):
    """Utilities to extract ligand neighbor mapping details from the exchange collections."""

    def __init__(self, cfgOb):
        self.__cfgOb = cfgOb

    def getLigandNeighbors(self):
        """Extract unique chemical component ids involved in neighbor interactions with each
        polymer and branched entity instance.

        Returns:
            dict: {'entryId_entityId':  [(chem_comp_id, isBound),...], }
        """
        return self.__extractLigandNeighbors()

    def __extractLigandNeighbors(self):
        """Extract unique chemical component ids involved in neighbor interactions with each
        polymer and branched entity instance."""
        try:
            databaseName = "pdbx_core"
            collectionName = "pdbx_core_polymer_entity_instance"
            obEx = ObjectExtractor(
                self.__cfgOb,
                databaseName=databaseName,
                collectionName=collectionName,
                cacheFilePath=None,
                useCache=False,
                keyAttribute="rcsb_id",
                uniqueAttributes=["rcsb_id"],
                cacheKwargs=None,
                objectLimit=None,
                # selectionQuery={"rcsb_polymer_entity_annotation.type": annotationType},
                selectionQuery=None,
                selectionList=[
                    "rcsb_id",
                    "rcsb_polymer_entity_instance_container_identifiers.entry_id",
                    "rcsb_polymer_entity_instance_container_identifiers.entity_id",
                    "rcsb_polymer_entity_instance_container_identifiers.asym_id",
                    "rcsb_ligand_neighbors.ligand_comp_id",
                    "rcsb_ligand_neighbors.ligand_is_bound",
                ],
            )
            eCount = obEx.getCount()
            logger.info("Total neighbor count (%d)", eCount)
            rD = {}
            objD = obEx.getObjects()
            for _, peiD in objD.items():
                try:
                    entryId = peiD["rcsb_polymer_entity_instance_container_identifiers"]["entry_id"]
                    entityId = peiD["rcsb_polymer_entity_instance_container_identifiers"]["entity_id"]
                    ky = entryId + "_" + entityId
                    for lnD in peiD["rcsb_ligand_neighbors"] if "rcsb_ligand_neighbors" in peiD else []:
                        if "ligand_comp_id" in lnD and "ligand_is_bound" in lnD:
                            rD.setdefault(ky, set()).add((lnD["ligand_comp_id"], lnD["ligand_is_bound"]))
                        else:
                            logger.warning("%s %s missing details lnD %r", entryId, entityId, lnD)
                except Exception as e:
                    logger.exception("Failing with %s", str(e))
            rD = {k: list(v) for k, v in rD.items()}
            logger.info("Unique instance %d", len(rD))
            return rD
        except Exception as e:
            logger.exception("Failing with %s", str(e))



##
# File:    PubChemEtlWorkflowTests.py
# Author:  J. Westbrook
# Date:    29-Jul-2020
#
# Updates:
#
##
"""
Tests for PubChem ETL workflow methods
"""

__docformat__ = "google en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"

import logging
import os
import platform
import resource
import time
import unittest

from rcsb.exdb.wf.PubChemEtlWorkflow import PubChemEtlWorkflow

logging.basicConfig(level=logging.INFO, format="%(asctime)s [%(levelname)s]-%(module)s.%(funcName)s: %(message)s")
logger = logging.getLogger()

HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(os.path.dirname(HERE)))


class PubChemEtlWorkflowTests(unittest.TestCase):
    def __init__(self, methodName="runTest"):
        super(PubChemEtlWorkflowTests, self).__init__(methodName)
        self.__verbose = True

    def setUp(self):
        #
        self.__dataPath = os.path.join(HERE, "test-data")
        self.__cachePath = os.path.join(HERE, "test-output", "CACHE")
        #
        # Site configuration used for database resource access -
        # self.__mockTopPath = os.path.join(TOPDIR, "rcsb", "mock-data")
        self.__configPath = os.path.join(TOPDIR, "rcsb", "mock-data", "config", "dbload-setup-example.yml")
        self.__configName = "site_info_configuration"
        #
        # These are test source files for chemical component/BIRD indices
        self.__ccUrlTarget = os.path.join(self.__dataPath, "components-abbrev.cif")
        self.__birdUrlTarget = os.path.join(self.__dataPath, "prdcc-abbrev.cif")
        self.__ccFileNamePrefix = "cc-abbrev"
        #
        self.__startTime = time.time()
        logger.debug("Starting %s at %s", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()))

    def tearDown(self):
        unitS = "MB" if platform.system() == "Darwin" else "GB"
        rusageMax = resource.getrusage(resource.RUSAGE_SELF).ru_maxrss
        logger.info("Maximum resident memory size %.4f %s", rusageMax / 10 ** 6, unitS)
        endTime = time.time()
        logger.info("Completed %s at %s (%.4f seconds)", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()), endTime - self.__startTime)

    def testAUpdateIndex(self):
        """Test case - build CCD/BIRD search indices and search for PubChem matches."""
        try:
            #  -- Update local chemical indices and  create PubChem mapping index ---

            pcewP = PubChemEtlWorkflow(configPath=self.__configPath, configName=self.__configName, cachePath=self.__cachePath)
            ok = pcewP.updateMatchedIndex(
                ccUrlTarget=self.__ccUrlTarget,
                birdUrlTarget=self.__birdUrlTarget,
                ccFileNamePrefix=self.__ccFileNamePrefix,
                numProc=4,
                rebuildChemIndices=True,
            )
            self.assertTrue(ok)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def testBDump(self):
        """Test case - dump current stored state"""
        try:
            #  --
            pcewP = PubChemEtlWorkflow(configPath=self.__configPath, configName=self.__configName, cachePath=self.__cachePath)
            ok = pcewP.dump()
            self.assertTrue(ok)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def testCRestore(self):
        """Test case - restore object store from the prior dump"""
        try:
            #  --
            pcewP = PubChemEtlWorkflow(configPath=self.__configPath, configName=self.__configName, cachePath=self.__cachePath)
            ok = pcewP.restore()
            self.assertTrue(ok)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def testDUpdateData(self):
        """Test case - update corresponding data and generate corresponding identifiers."""
        try:
            #  --
            pcewP = PubChemEtlWorkflow(configPath=self.__configPath, configName=self.__configName, cachePath=self.__cachePath)
            ok = pcewP.updateMatchedData()
            self.assertTrue(ok)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()


def pubChemEtlWorkflowSuite():
    suiteSelect = unittest.TestSuite()
    suiteSelect.addTest(PubChemEtlWorkflowTests("testAUpdateIndex"))
    suiteSelect.addTest(PubChemEtlWorkflowTests("testBDump"))
    suiteSelect.addTest(PubChemEtlWorkflowTests("testCRestore"))
    suiteSelect.addTest(PubChemEtlWorkflowTests("testDUpdateData"))

    return suiteSelect


if __name__ == "__main__":
    mySuite = pubChemEtlWorkflowSuite()
    unittest.TextTestRunner(verbosity=2).run(mySuite)



##
# File:    CitationAdapterTests.py
# Author:  J. Westbrook
# Date:    22-Nov-2019
#
# Updates:
#
##
"""
Tests of reference sequence assignment adapter.
"""

__docformat__ = "google en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"

import logging
import os
import platform
import resource
import time
import unittest

from rcsb.exdb.citation.CitationAdapter import CitationAdapter
from rcsb.exdb.utils.ObjectTransformer import ObjectTransformer
from rcsb.utils.citation.CitationReferenceProvider import CitationReferenceProvider
from rcsb.utils.citation.JournalTitleAbbreviationProvider import JournalTitleAbbreviationProvider
from rcsb.utils.config.ConfigUtil import ConfigUtil

logging.basicConfig(level=logging.INFO, format="%(asctime)s [%(levelname)s]-%(module)s.%(funcName)s: %(message)s")
logger = logging.getLogger()

HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(os.path.dirname(HERE)))


class CitationAdapterTests(unittest.TestCase):
    def __init__(self, methodName="runTest"):
        super(CitationAdapterTests, self).__init__(methodName)
        self.__verbose = True

    def setUp(self):
        #
        self.__mockTopPath = os.path.join(TOPDIR, "rcsb", "mock-data")
        configPath = os.path.join(TOPDIR, "rcsb", "mock-data", "config", "dbload-setup-example.yml")
        configName = "site_info_configuration"
        self.__cfgOb = ConfigUtil(configPath=configPath, defaultSectionName=configName, mockTopPath=self.__mockTopPath)
        #
        self.__resourceName = "MONGO_DB"
        self.__cachePath = os.path.join(TOPDIR, "CACHE", "cit_ref")
        self.__testEntityCacheKwargs = {"fmt": "json", "indent": 3}
        self.__fetchLimit = None
        #
        self.__startTime = time.time()
        logger.debug("Starting %s at %s", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()))

    def tearDown(self):
        unitS = "MB" if platform.system() == "Darwin" else "GB"
        rusageMax = resource.getrusage(resource.RUSAGE_SELF).ru_maxrss
        logger.info("Maximum resident memory size %.4f %s", rusageMax / 10 ** 6, unitS)
        endTime = time.time()
        logger.info("Completed %s at %s (%.4f seconds)", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()), endTime - self.__startTime)

    def testCitationAdapter(self):
        """Test case - create and read cache reference sequences assignments and related data."""
        try:
            databaseName = "pdbx_core"
            collectionName = "pdbx_core_entry"
            useCache = False
            #
            crP = CitationReferenceProvider(cachePath=self.__cachePath, useCache=useCache)
            ok = crP.testCache()
            self.assertTrue(ok)
            jtaP = JournalTitleAbbreviationProvider(cachePath=self.__cachePath, useCache=useCache)
            ok = jtaP.testCache()
            self.assertTrue(ok)
            #
            ca = CitationAdapter(crP, jtaP)
            obTr = ObjectTransformer(self.__cfgOb, objectAdapter=ca)
            ok = obTr.doTransform(databaseName=databaseName, collectionName=collectionName, fetchLimit=self.__fetchLimit)
            self.assertTrue(ok)

        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()


def citationAdapterSuite():
    suiteSelect = unittest.TestSuite()
    suiteSelect.addTest(CitationAdapterTests("testCitationAdapter"))
    return suiteSelect


if __name__ == "__main__":
    mySuite = citationAdapterSuite()
    unittest.TextTestRunner(verbosity=2).run(mySuite)



##
#  File:           EntryInfoProvider.py
#  Date:           22-Sep-2021 jdw
#
#  Updated:
#
##
"""
Accessors for entry-level annotations.

"""

import logging
import os.path
import time

from rcsb.exdb.utils.ObjectExtractor import ObjectExtractor
from rcsb.utils.io.MarshalUtil import MarshalUtil
from rcsb.utils.io.StashableBase import StashableBase

logger = logging.getLogger(__name__)


class EntryInfoProvider(StashableBase):
    """Accessors and generators for entry-level annotations."""

    def __init__(self, **kwargs):
        #
        self.__version = "0.50"
        cachePath = kwargs.get("cachePath", ".")
        useCache = kwargs.get("useCache", True)
        self.__dirName = "rcsb_entry_info"
        self.__dirPath = os.path.join(cachePath, self.__dirName)
        super(EntryInfoProvider, self).__init__(cachePath, [self.__dirName])
        #
        self.__mU = MarshalUtil(workPath=self.__dirPath)
        self.__entryInfoD = self.__reload(fmt="json", useCache=useCache)
        #

    def testCache(self, minCount=1):
        if minCount == 0:
            return True
        if self.__entryInfoD and minCount and "entryInfo" in self.__entryInfoD and len(self.__entryInfoD["entryInfo"]) > minCount:
            logger.info("Entry annotations for (%d) entries", len(self.__entryInfoD["entryInfo"]))
            return True
        return False

    def getEntryInfo(self, entryId):
        """Return a dictionary of entry-level annotations.

        Returns:
            (dict): of entry-level annotations
        """
        try:
            return self.__entryInfoD["entryInfo"][entryId.upper()] if entryId.upper() in self.__entryInfoD["entryInfo"] else {}
        except Exception as e:
            logger.error("Failing with %r", str(e))
        return {}

    def getEntriesByPolymerEntityCount(self, count):
        oL = []
        try:
            for entryId, eD in self.__entryInfoD["entryInfo"].items():
                if eD["polymer_entity_count"] == count:
                    oL.append(entryId)
        except Exception as e:
            logger.error("Failing with %r", str(e))
        return oL

    def __getEntryInfoFilePath(self, fmt="json"):
        baseFileName = "entry_info_details"
        fExt = ".json" if fmt == "json" else ".pic"
        fp = os.path.join(self.__dirPath, baseFileName + fExt)
        return fp

    def update(self, cfgOb, fmt="json", indent=3):
        """Update branched entity glycan accession mapping cache.

        Args:
            cfgObj (object): ConfigInfo() object instance

        Returns:
            (bool): True for success for False otherwise
        """
        ok = False
        try:
            entryInfoD = self.__updateEntryInfo(cfgOb)

            logger.info("Got entry_info for (%d)", len(entryInfoD))
            #
            tS = time.strftime("%Y %m %d %H:%M:%S", time.localtime())
            self.__entryInfoD = {"version": self.__version, "created": tS, "entryInfo": entryInfoD}
            #
            infoFilePath = self.__getEntryInfoFilePath(fmt=fmt)
            kwargs = {"indent": indent} if fmt == "json" else {}
            ok = self.__mU.doExport(infoFilePath, self.__entryInfoD, fmt=fmt, **kwargs)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return ok

    def reload(self):
        """Reload from the current cache file."""
        ok = False
        try:
            self.__entryInfoD = self.__reload(fmt="json", useCache=True)
            ok = self.__entryInfoD is not None
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return ok

    def __reload(self, fmt="json", useCache=True):
        entryInfoFilePath = self.__getEntryInfoFilePath(fmt=fmt)
        tS = time.strftime("%Y %m %d %H:%M:%S", time.localtime())
        pcD = {"version": self.__version, "created": tS, "identifiers": {}}

        if useCache and self.__mU.exists(entryInfoFilePath):
            logger.info("Reading entry-info cached path %r", entryInfoFilePath)
            pcD = self.__mU.doImport(entryInfoFilePath, fmt=fmt)
        return pcD

    def __updateEntryInfo(self, cfgOb):
        """Get entry_info data"""
        rD = {}
        try:
            obEx = ObjectExtractor(
                cfgOb,
                databaseName="pdbx_core",
                collectionName="pdbx_core_entry",
                useCache=False,
                keyAttribute="entry",
                uniqueAttributes=["rcsb_id"],
                selectionQuery={},
                selectionList=["rcsb_id", "rcsb_entry_info.polymer_entity_count"],
            )
            #
            eCount = obEx.getCount()
            logger.info("Entry count is %d", eCount)

            objD = obEx.getObjects()
            for _, eD in objD.items():
                rcsbId = eD["rcsb_id"]
                try:
                    rD[rcsbId] = eD["rcsb_entry_info"]
                except Exception:
                    pass
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return rD



##
# File:    PdbxLoaderFixture.py
# Author:  J. Westbrook
# Date:    4-Sep-2019
# Version: 0.001
#
# Updates:
#
##
"""
Fixture for loading the chemical reference and pdbx_core collections in a loca mongo instance.

"""

__docformat__ = "google en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"

# import glob
import logging
import os
import platform
import resource
import time
import unittest

from rcsb.db.mongo.DocumentLoader import DocumentLoader
from rcsb.db.mongo.PdbxLoader import PdbxLoader
from rcsb.utils.config.ConfigUtil import ConfigUtil
# from rcsb.utils.io.FileUtil import FileUtil

logging.basicConfig(level=logging.INFO, format="%(asctime)s [%(levelname)s]-%(module)s.%(funcName)s: %(message)s")
logger = logging.getLogger()
logger.setLevel(logging.INFO)

HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(os.path.dirname(HERE)))


class PdbxLoaderFixture(unittest.TestCase):

    def __init__(self, methodName="runTest"):
        super(PdbxLoaderFixture, self).__init__(methodName)
        self.__verbose = True

    def setUp(self):
        #
        #
        self.__isMac = platform.system() == "Darwin"
        self.__excludeType = None if self.__isMac else "optional"
        self.__mockTopPath = os.path.join(TOPDIR, "rcsb", "mock-data")
        configPath = os.path.join(TOPDIR, "rcsb", "mock-data", "config", "dbload-setup-example.yml")
        # configPath = os.path.join(TOPDIR, "rcsb", "mock-data", "config", "dbload-setup-example-local.yml")
        # To Do: Investigate why GitUtil sometimes gives divergence error when using 'DISCOVERY_MODE: remote', but not with 'local':
        #            stderr: 'fatal: Need to specify how to reconcile divergent branches.'
        #        Behavior isn't entirely predictable, since it happens sometimes but not all the time.
        #        To fully debug, will need to add more logging statements to GitUtil, StashableBase, & StashUtil (in rcsb.utils.io)
        #        Or, can try to resolve error directly by specifying how to reconcile diverent branches in git.Repo class.
        configName = "site_info_configuration"
        self.__cfgOb = ConfigUtil(configPath=configPath, defaultSectionName=configName, mockTopPath=self.__mockTopPath)
        #
        self.__resourceName = "MONGO_DB"
        self.__failedFilePath = os.path.join(HERE, "test-output", "failed-list.txt")
        self.__cachePath = os.path.join(TOPDIR, "CACHE")
        self.__readBackCheck = True
        self.__numProc = 2
        self.__chunkSize = 10
        self.__fileLimit = 38
        self.__documentStyle = "rowwise_by_name_with_cardinality"
        #
        self.__birdChemCompCoreIdList = [
            "PRD_000010",
            "PRD_000060",
            "PRD_000220",
            "PRD_000882",
            "PRD_000154",
            "PRD_000877",
            "PRD_000198",
            "PRD_000009",
            "PRD_000979",
            "PRDCC_000010",
            "PRDCC_000220",
            "PRDCC_000882",
            "PRDCC_000154",
            "PRDCC_000198",
            "PRDCC_000009",
            "FAM_000010",
            "FAM_000210",
            "FAM_000220",
            "FAM_000001",
            "FAM_000391",
            "FAM_000093",
            "FAM_000084",
            "FAM_000016",
            "FAM_000336",
            "1G1",
            "2RT",
            "2XL",
            "2XN",
            "ATP",
            "BJA",
            "BM3",
            "CNC",
            "DAL",
            "DDZ",
            "DHA",
            "DSN",
            "GTP",
            "HKL",
            "NAC",
            "NAG",
            "NND",
            "PTR",
            "SEP",
            "SMJ",
            "STL",
            "UNK",
            "UNX",
            "UVL",
        ]
        #
        self.__pdbIdList = [
            "1ah1",
            "1b5f",
            "1bmv",
            "1c58",
            "1dsr",
            "1dul",
            "1kqe",
            "1o3q",
            "1sfo",
            "2hw3",
            "2hyv",
            "2osl",
            "2voo",
            "2wmg",
            "3ad7",
            "3hya",
            "3iyd",
            "3mbg",
            "3rer",
            "3vd8",
            "3vfj",
            "3x11",
            "3ztj",
            "4e2o",
            "4en8",
            "4mey",
            "5eu8",
            "5kds",
            "5tm0",
            "5vh4",
            "5vp2",
            "6fsz",
            "6lu7",
            "6nn7",
            "6q20",
            "6rfk",
            "6rku",
            "6yrq",
        ]
        self.__ldList = [
            {
                "databaseName": "bird_chem_comp_core",
                "collectionNameList": None,
                "loadType": "full",
                "mergeContentTypes": None,
                "validationLevel": "min",
                "inputIdCodeList": self.__birdChemCompCoreIdList
            },
            {
                "databaseName": "pdbx_core",
                "collectionNameList": None,
                "loadType": "full",
                "mergeContentTypes": ["vrpt"],
                "validationLevel": "min",
                "inputIdCodeList": self.__pdbIdList
            },
            # {
            #     "databaseName": "pdbx_comp_model_core",
            #     "collectionNameList": None,
            #     "loadType": "full",
            #     "mergeContentTypes": None,
            #     "validationLevel": "min",
            #     "inputIdCodeList": None
            # },
        ]
        #
        # self.__modelFixture()
        self.__startTime = time.time()
        logger.debug("Starting %s at %s", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()))

    def tearDown(self):
        unitS = "MB" if platform.system() == "Darwin" else "GB"
        rusageMax = resource.getrusage(resource.RUSAGE_SELF).ru_maxrss
        logger.info("Maximum resident memory size %.4f %s", rusageMax / 10 ** 6, unitS)
        endTime = time.time()
        logger.info("Completed %s at %s (%.4f seconds)", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()), endTime - self.__startTime)

    # def __modelFixture(self):
    #     fU = FileUtil()
    #     modelSourcePath = os.path.join(self.__mockTopPath, "AF")
    #     for iPath in glob.iglob(os.path.join(modelSourcePath, "*.cif.gz")):
    #         fn = os.path.basename(iPath)
    #         uId = fn.split("-")[1]
    #         h3 = uId[-2:]
    #         h2 = uId[-4:-2]
    #         h1 = uId[-6:-4]
    #         oPath = os.path.join(self.__cachePath, "computed-models", h1, h2, h3, fn)
    #         fU.put(iPath, oPath)

    def testPdbxLoader(self):
        #
        for ld in self.__ldList:
            self.__pdbxLoaderWrapper(**ld)

    def __pdbxLoaderWrapper(self, **kwargs):
        """Wrapper for the PDBx loader module"""
        try:
            logger.info("Loading %s", kwargs["databaseName"])
            mw = PdbxLoader(
                self.__cfgOb,
                cachePath=self.__cachePath,
                resourceName=self.__resourceName,
                numProc=self.__numProc,
                chunkSize=self.__chunkSize,
                fileLimit=kwargs.get("fileLimit", self.__fileLimit),
                verbose=self.__verbose,
                readBackCheck=self.__readBackCheck,
                maxStepLength=2000,
                useSchemaCache=True,
                rebuildSchemaFlag=False,
            )
            ok = mw.load(
                kwargs["databaseName"],
                collectionLoadList=kwargs["collectionNameList"],
                loadType=kwargs["loadType"],
                inputPathList=None,
                inputIdCodeList=kwargs["inputIdCodeList"],
                styleType=self.__documentStyle,
                dataSelectors=["PUBLIC_RELEASE"],
                failedFilePath=self.__failedFilePath,
                saveInputFileListPath=None,
                pruneDocumentSize=None,
                logSize=False,
                validationLevel=kwargs["validationLevel"],
                mergeContentTypes=kwargs["mergeContentTypes"],
                useNameFlag=False,
                providerTypeExclude=self.__excludeType,
                restoreUseGit=True,
                restoreUseStash=False,
            )
            self.assertTrue(ok)
            ok = self.__loadStatus(mw.getLoadStatus())
            self.assertTrue(ok)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def __loadStatus(self, statusList):
        sectionName = "data_exchange_configuration"
        dl = DocumentLoader(
            self.__cfgOb,
            self.__cachePath,
            resourceName=self.__resourceName,
            numProc=self.__numProc,
            chunkSize=self.__chunkSize,
            documentLimit=None,
            verbose=self.__verbose,
            readBackCheck=self.__readBackCheck,
        )
        #
        databaseName = self.__cfgOb.get("DATABASE_NAME", sectionName=sectionName)
        collectionName = self.__cfgOb.get("COLLECTION_UPDATE_STATUS", sectionName=sectionName)
        ok = dl.load(databaseName, collectionName, loadType="append", documentList=statusList, indexAttributeList=["update_id", "database_name", "object_name"], keyNames=None)
        return ok


def mongoLoadPdbxSuite():
    suiteSelect = unittest.TestSuite()
    suiteSelect.addTest(PdbxLoaderFixture("testPdbxLoader"))
    return suiteSelect


if __name__ == "__main__":
    mySuite = mongoLoadPdbxSuite()
    unittest.TextTestRunner(verbosity=2).run(mySuite)



##
#  File:           LigandNeighborMappingProvider.py
#  Date:           28-Jun-2021 jdw
#
#  Updated:
#
##
"""
Accessors for essential ligand neighbor mapping details associated with polymer and branched
entity instances.
"""

import datetime
import logging
import os.path
import time

from rcsb.utils.io.FileUtil import FileUtil
from rcsb.utils.io.MarshalUtil import MarshalUtil
from rcsb.utils.io.StashableBase import StashableBase
from rcsb.exdb.seq.LigandNeighborMappingExtractor import LigandNeighborMappingExtractor

logger = logging.getLogger(__name__)


class LigandNeighborMappingProvider(StashableBase):
    """Accessors for essential ligand neighbor mapping details associated with polymer and branched
    entity instances."""

    def __init__(self, cachePath, useCache=True):
        #
        self.__cachePath = cachePath
        self.__useCache = useCache
        self.__dirName = "ligand-neighbor-mapping"
        super(LigandNeighborMappingProvider, self).__init__(self.__cachePath, [self.__dirName])
        self.__dirPath = os.path.join(self.__cachePath, self.__dirName)
        #
        self.__mU = MarshalUtil(workPath=self.__dirPath)
        self.__mapD = self.__reload(self.__dirPath, useCache)
        #

    def testCache(self, minCount=0):
        logger.info("Cached ligand neighbor mapping count %d", len(self.__mapD["mapping"]) if "mapping" in self.__mapD else 0)
        if minCount == 0 or self.__mapD and "mapping" in self.__mapD and len(self.__mapD["mapping"]) >= minCount:
            return True
        else:
            return False

    def getLigandNeighbors(self, rcsbEntityId):
        """Get the unique list of ligand neighbors for the input polymer or branched entity instance.

        Args:
            rcsbEntityId (str): entryId '_' entityId

        Returns:
            list: [chem_comp_id, ... ]
        """
        try:
            return list(set([t[0] for t in self.__mapD["mapping"][rcsbEntityId.upper()]]))
        except Exception:
            return []

    def __reload(self, dirPath, useCache):
        startTime = time.time()
        retD = {}
        ok = False
        mappingPath = self.__getMappingDataPath()
        #
        logger.info("useCache %r mappingPath %r", useCache, mappingPath)
        if useCache and self.__mU.exists(mappingPath):
            retD = self.__mU.doImport(mappingPath, fmt="json")
            ok = True
        else:
            fU = FileUtil()
            fU.mkdir(dirPath)
        # ---
        num = len(retD["mapping"]) if "mapping" in retD else 0
        logger.info("Completed ligand mapping reload (%d) with status (%r) at %s (%.4f seconds)", num, ok, time.strftime("%Y %m %d %H:%M:%S", time.localtime()), time.time() - startTime)
        return retD

    def __getMappingDataPath(self):
        return os.path.join(self.__dirPath, "ligand-neighbor-mapping-data.json")

    def fetchLigandNeighborMapping(self, cfgOb):
        """Fetch ligand neighbor mapping details

        Args:
            cfgOb (obj): instance configuration class ConfigUtil()

        Returns:
            bool: True for success or False otherwise
        """
        try:
            lnmEx = LigandNeighborMappingExtractor(cfgOb)
            lnD = lnmEx.getLigandNeighbors()
            fp = self.__getMappingDataPath()
            tS = datetime.datetime.now().isoformat()
            vS = datetime.datetime.now().strftime("%Y-%m-%d")
            ok = self.__mU.doExport(fp, {"version": vS, "created": tS, "mapping": lnD}, fmt="json", indent=3)
            return ok
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return False



##
# File: CitationAdapter.py
# Date: 21-Nov-2019  jdw
#
# Selected utilities to update entry citations in the core_entry collection.
#
# Updates:
#
##
__docformat__ = "google en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"

import copy
import logging
from string import capwords

from rcsb.exdb.utils.ObjectAdapterBase import ObjectAdapterBase

logger = logging.getLogger(__name__)


class CitationAdapter(ObjectAdapterBase):
    """Selected utilities to update entry citations in the core_entry collection."""

    def __init__(self, citationReferenceProvider, journalTitleAbbreviationProvider):
        super(CitationAdapter, self).__init__()
        #
        self.__crP = citationReferenceProvider
        self.__jtaP = journalTitleAbbreviationProvider

    def filter(self, obj, **kwargs):
        isTestMode = True
        if isTestMode:
            _, _ = self.__filter(copy.deepcopy(obj))
            return True, obj
        else:
            return self.__filter(obj)

    def __filter(self, obj):
        ok = True
        try:
            rcsbId = obj["rcsb_id"]
            if "citation" in obj:
                for citObj in obj["citation"]:
                    if citObj["id"].upper() != "PRIMARY":
                        continue
                    issn = citObj["journal_id_ISSN"] if "journal_id_ISSN" in citObj else None
                    curAbbrev = citObj["journal_abbrev"] if "journal_abbrev" in citObj else None
                    revAbbrev = self.__updateJournalAbbreviation(rcsbId, issn, curAbbrev)
                    logger.debug("%s: revised: %r current: %r", rcsbId, revAbbrev, curAbbrev)

        except Exception as e:
            ok = False
            logger.exception("Filter adapter failing with error with %s", str(e))
        #
        return ok, obj

    def __updateJournalAbbreviation(self, rcsbId, issn, curAbbrev):
        try:
            revAbbrev = None
            if issn:
                medlineAbbrev = self.__crP.getMedlineJournalAbbreviation(issn)
                # medlineIsoAbbrev = self.__crP.getMedlineJournalIsoAbbreviation(issn)
                crIssn = issn.replace("-", "")
                crTitle = self.__crP.getCrossRefJournalTitle(crIssn)
                #
                revAbbrev = medlineAbbrev
                if not medlineAbbrev and not crTitle:
                    logger.debug("%s: missing information for issn %r curAbbrev %r", rcsbId, issn, curAbbrev)
                    revAbbrev = capwords(curAbbrev.replace(".", " "))
                elif not medlineAbbrev:
                    revAbbrev = self.__jtaP.getJournalAbbreviation(crTitle, usePunctuation=False)
            else:
                if curAbbrev.upper() in ["TO BE PUBLISHED", "IN PREPARATION"]:
                    revAbbrev = "To be published"
                elif curAbbrev.upper().startswith("THESIS"):
                    revAbbrev = "Thesis"
                else:
                    revAbbrev = capwords(curAbbrev.replace(".", " "))
                    logger.debug("%r: missing issn and non-standard abbrev for %r", rcsbId, curAbbrev)

                if not curAbbrev:
                    logger.info("%r: missing issn and journal abbrev", rcsbId)
                #
            logger.debug("%s: revised: %r current: %r", rcsbId, revAbbrev, curAbbrev)
        except Exception as e:
            logger.exception("Failing on %r %r %r with %r", rcsbId, issn, curAbbrev, str(e))

        return revAbbrev



##
# File: ReferenceSequenceAnnotationProvider.py
# Date: 14-Feb-2020  jdw
#
# Utilities to cache content required to update referencence sequence annotations.
#
# Updates:
#  25-May-2022 dwp Add error checking for SIFTS data loading
##
__docformat__ = "google en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"

import logging
import os
from collections import defaultdict

from rcsb.exdb.seq.ReferenceSequenceCacheProvider import ReferenceSequenceCacheProvider
from rcsb.utils.ec.EnzymeDatabaseProvider import EnzymeDatabaseProvider
from rcsb.utils.go.GeneOntologyProvider import GeneOntologyProvider
from rcsb.utils.io.IoUtil import getObjSize
from rcsb.utils.io.MarshalUtil import MarshalUtil
from rcsb.utils.seq.GlyGenProvider import GlyGenProvider
from rcsb.utils.seq.InterProProvider import InterProProvider
from rcsb.utils.seq.PfamProvider import PfamProvider
from rcsb.utils.seq.SiftsSummaryProvider import SiftsSummaryProvider
from rcsb.utils.seq.UniProtUtils import UniProtUtils

logger = logging.getLogger(__name__)


class ReferenceSequenceAnnotationProvider(object):
    """Utilities to cache content required to update referencence sequence annotations."""

    def __init__(self, cfgOb, databaseName, collectionName, polymerType, maxChunkSize=100, fetchLimit=None, numProc=2, expireDays=14, **kwargs):
        self.__cfgOb = cfgOb
        self.__mU = MarshalUtil()
        #
        self.__maxChunkSize = maxChunkSize
        self.__statusList = []
        #
        self.__ggP = self.__fetchGlyGenProvider(self.__cfgOb, self.__cfgOb.getDefaultSectionName(), **kwargs)
        self.__pfP = self.__fetchPfamProvider(self.__cfgOb, self.__cfgOb.getDefaultSectionName(), **kwargs)
        self.__ipP = self.__fetchInterProProvider(self.__cfgOb, self.__cfgOb.getDefaultSectionName(), **kwargs)
        self.__ssP = self.__fetchSiftsSummaryProvider(self.__cfgOb, self.__cfgOb.getDefaultSectionName(), **kwargs)
        self.__goP = self.__fetchGoProvider(self.__cfgOb, self.__cfgOb.getDefaultSectionName(), **kwargs)
        self.__ecP = self.__fetchEcProvider(self.__cfgOb, self.__cfgOb.getDefaultSectionName(), **kwargs)
        #
        self.__rsaP = ReferenceSequenceCacheProvider(
            self.__cfgOb, databaseName, collectionName, polymerType, siftsProvider=self.__ssP, maxChunkSize=maxChunkSize, numProc=numProc, fetchLimit=fetchLimit, expireDays=expireDays
        )
        self.__matchD = self.__rsaP.getMatchInfo()
        self.__refD = self.__rsaP.getRefData()
        self.__missingMatchedIdCodes = self.__rsaP.getMissingMatchedIdCodes()

    def goIdExists(self, goId):
        try:
            return self.__goP.exists(goId)
        except Exception as e:
            logger.exception("Failing for %r with %s", goId, str(e))
        return False

    def getGeneOntologyName(self, goId):
        try:
            return self.__goP.getName(goId)
        except Exception as e:
            logger.exception("Failing for %r with %s", goId, str(e))
        return None

    def getGeneOntologyLineage(self, goIdL):
        # "id"     "name"
        gL = []
        try:
            gTupL = self.__goP.getUniqueDescendants(goIdL)
            for gTup in gTupL:
                gL.append({"id": gTup[0], "name": gTup[1]})
        except Exception as e:
            logger.exception("Failing for %r with %s", goIdL, str(e))
        return gL

    def getGlyGenProvider(self):
        return self.__ggP

    def getPfamProvider(self):
        return self.__pfP

    def getPfamName(self, idCode):
        return self.__pfP.getDescription(idCode)

    def getInterProProvider(self):
        return self.__ipP

    def getInterProName(self, idCode):
        return self.__ipP.getDescription(idCode)

    def getInterProLineage(self, idCode):
        linL = []
        try:
            tupL = self.__ipP.getLineageWithNames(idCode)
            for tup in tupL:
                linL.append({"id": tup[0], "name": tup[1], "depth": tup[2]})
        except Exception as e:
            logger.exception("Failing for %r with %s", idCode, str(e))
        return linL

    def getEcProvider(self):
        return self.__ecP

    def getSiftsSummaryProvider(self):
        return self.__ssP

    def getMatchInfo(self):
        return self.__matchD

    def getRefData(self):
        return self.__refD

    def getDocuments(self, formatType="exchange"):
        fobj = UniProtUtils(saveText=False)
        exObjD = fobj.reformat(self.__refD, formatType=formatType)
        return list(exObjD.values())

    def getRefDataCount(self):
        return len(self.__refD)

    def testCache(self, minMatchPrimaryPercent=None, logSizes=False, minMissing=0):
        okC = True
        logger.info("Reference sequence cache lengths: matchD %d refD %d", len(self.__matchD), len(self.__refD))
        ok = bool(self.__matchD and self.__refD and self.__ssP and self.__missingMatchedIdCodes <= minMissing)
        #
        numRef = len(self.__matchD)
        countD = defaultdict(int)
        logger.info("Match dictionary length %d", len(self.__matchD))
        for _, mD in self.__matchD.items():
            if "matched" in mD:
                countD[mD["matched"]] += 1
        logger.info("Reference length %d match length %d coverage %r", len(self.__refD), len(self.__matchD), countD.items())
        if minMatchPrimaryPercent:
            try:
                okC = 100.0 * float(countD["primary"]) / float(numRef) > minMatchPrimaryPercent
            except Exception:
                okC = False
            logger.info("Primary reference match percent test status %r", okC)
        #
        if logSizes:
            logger.info(
                "SIFTS %.2f GO %.2f EC %.2f RefMatchD %.2f RefD %.2f",
                getObjSize(self.__ssP) / 1000000.0,
                getObjSize(self.__goP) / 1000000.0,
                getObjSize(self.__ecP) / 1000000.0,
                getObjSize(self.__matchD) / 1000000.0,
                getObjSize(self.__refD) / 1000000.0,
            )
        return ok and okC

    def __fetchSiftsSummaryProvider(self, cfgOb, configName, **kwargs):
        abbreviated = kwargs.get("siftsAbbreviated", "TEST")
        cachePath = kwargs.get("cachePath", ".")
        cacheKwargs = kwargs.get("cacheKwargs", {"fmt": "pickle"})
        useCache = kwargs.get("useCache", True)
        #
        siftsSummaryDataPath = cfgOb.getPath("SIFTS_SUMMARY_DATA_PATH", sectionName=configName)
        if siftsSummaryDataPath.lower().startswith("http"):
            srcDirPath = siftsSummaryDataPath
        else:
            srcDirPath = os.path.join(cachePath, siftsSummaryDataPath)
        cacheDirPath = os.path.join(cachePath, cfgOb.get("SIFTS_SUMMARY_CACHE_DIR", sectionName=configName))
        logger.debug("ssP %r %r", srcDirPath, cacheDirPath)
        ssP = SiftsSummaryProvider(srcDirPath=srcDirPath, cacheDirPath=cacheDirPath, useCache=useCache, abbreviated=abbreviated, cacheKwargs=cacheKwargs)
        ok = ssP.testCache()
        if not ok:
            logger.error("Failed to refetch SIFTS summary data using srcDirPath %s, cacheDirPath %s", srcDirPath, cacheDirPath)
            return None
        logger.debug("SIFTS cache status %r", ok)
        logger.debug("ssP entry count %d", ssP.getEntryCount())
        return ssP

    def __fetchGoProvider(self, cfgOb, configName, **kwargs):
        cachePath = kwargs.get("cachePath", ".")
        useCache = kwargs.get("useCache", True)
        #
        cacheDirPath = os.path.join(cachePath, cfgOb.get("EXDB_CACHE_DIR", sectionName=configName))
        logger.debug("goP %r %r", cacheDirPath, useCache)
        goP = GeneOntologyProvider(goDirPath=cacheDirPath, useCache=useCache)
        ok = goP.testCache()
        logger.debug("Gene Ontology (%r) root node count %r", ok, goP.getRootNodes())
        return goP

    def __fetchEcProvider(self, cfgOb, configName, **kwargs):
        cachePath = kwargs.get("cachePath", ".")
        useCache = kwargs.get("useCache", True)
        #
        cacheDirPath = os.path.join(cachePath, cfgOb.get("ENZYME_CLASSIFICATION_CACHE_DIR", sectionName=configName))
        logger.debug("ecP %r %r", cacheDirPath, useCache)
        ecP = EnzymeDatabaseProvider(enzymeDirPath=cacheDirPath, useCache=useCache)
        ok = ecP.testCache()
        logger.debug("Enzyme cache status %r", ok)
        return ecP

    def __fetchGlyGenProvider(self, cfgOb, configName, **kwargs):
        _ = cfgOb
        _ = configName
        cachePath = kwargs.get("cachePath", ".")
        useCache = kwargs.get("useCache", True)
        ggP = GlyGenProvider(cachePath=cachePath, useCache=useCache)
        ok = ggP.testCache()
        return ggP if ok else None

    def __fetchPfamProvider(self, cfgOb, configName, **kwargs):
        _ = cfgOb
        _ = configName
        cachePath = kwargs.get("cachePath", ".")
        useCache = kwargs.get("useCache", True)
        pfP = PfamProvider(cachePath=cachePath, useCache=useCache)
        ok = pfP.testCache()
        return pfP if ok else None

    def __fetchInterProProvider(self, cfgOb, configName, **kwargs):
        _ = cfgOb
        _ = configName
        cachePath = kwargs.get("cachePath", ".")
        useCache = kwargs.get("useCache", True)
        ipP = InterProProvider(cachePath=cachePath, useCache=useCache)
        ok = ipP.testCache()
        return ipP if ok else None



##
# File: PubChemEtlWrapper.py
# Date: 19-Jul-2029  jdw
#
#
# Updates:
#
#
##
__docformat__ = "google en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"

import logging
import os

from rcsb.exdb.chemref.PubChemDataCacheProvider import PubChemDataCacheProvider
from rcsb.exdb.chemref.PubChemIndexCacheProvider import PubChemIndexCacheProvider
from rcsb.utils.chemref.PubChemProvider import PubChemProvider

logger = logging.getLogger(__name__)


class PubChemEtlWrapper(object):
    """Workflow wrapper for updating chemical component/BIRD to PubChem mapping and related PubChem reference data."""

    def __init__(self, cfgOb, cachePath, **kwargs):
        self.__cfgOb = cfgOb
        self.__configName = self.__cfgOb.getDefaultSectionName()
        self.__cachePath = cachePath
        self.__dirPath = os.path.join(self.__cachePath, "PubChem")
        #
        self.__stashRemotePrefix = kwargs.get("stashRemotePrefix", None)
        #
        self.__pcicP = PubChemIndexCacheProvider(self.__cfgOb, self.__cachePath)
        self.__pcdcP = PubChemDataCacheProvider(self.__cfgOb, self.__cachePath)
        self.__pcP = PubChemProvider(cachePath=self.__cachePath)
        #
        self.__identifierD = None
        #

    def reloadDump(self, contentType="index"):
        """Reload the input content type in the data store from saved object store dump.

        Args:
            contentType (str): target content to restore (data|index)

        Returns:
            (int): number of records in restored collection.
        """
        numRecords = 0
        if contentType.lower() == "index":
            numRecords = self.__pcicP.reloadDump()
        elif contentType.lower() == "data":
            numRecords = self.__pcdcP.reloadDump()
        return numRecords

    def dump(self, contentType):
        """Dump PubChem content from the object store.

        Args:
            contentType (str): target content to restore (data|index)

        Returns:
            (bool): True for success or False otherwise
        """
        ok = False
        if contentType.lower() == "index":
            ok = self.__pcicP.dump()
        elif contentType.lower() == "data":
            ok = self.__pcdcP.dump()
        elif contentType.lower() == "identifiers":
            ok = self.__dumpIdentifiers()

        return ok

    def toStash(self, contentType, useGit=True, useStash=True):
        """Store PubChem extracted content () on the remote stash storage resource.

        Args:
            contentType (str): target content to stash (data|index|identifiers)
        Returns:
            (bool): True for success or False otherwise
        """
        if contentType.lower() == "index":
            return self.__pcicP.backup(self.__cfgOb, self.__configName, remotePrefix=self.__stashRemotePrefix, useGit=useGit, useStash=useStash)
        elif contentType.lower() == "data":
            return self.__pcdcP.backup(self.__cfgOb, self.__configName, remotePrefix=self.__stashRemotePrefix, useGit=useGit, useStash=useStash)
        elif contentType.lower() == "identifiers":
            return self.__pcP.backup(self.__cfgOb, self.__configName, remotePrefix=self.__stashRemotePrefix, useGit=useGit, useStash=useStash)
        return False

    def fromStash(self, contentType, useStash=True, useGit=True):
        """Fetch PubChem extracted content from the remote stash storage resource.

        Args:
            contentType (str): target content to fetch (data|index)
        Returns:
            (bool): True for success or False otherwise
        """
        if contentType.lower() == "index":
            return self.__pcicP.restore(self.__cfgOb, self.__configName, remotePrefix=self.__stashRemotePrefix, useGit=useGit, useStash=useStash)
        elif contentType.lower() == "data":
            return self.__pcdcP.restore(self.__cfgOb, self.__configName, remotePrefix=self.__stashRemotePrefix, useGit=useGit, useStash=useStash)
        elif contentType.lower() == "identifiers":
            return self.__pcdcP.restore(self.__cfgOb, self.__configName, remotePrefix=self.__stashRemotePrefix, useGit=useGit, useStash=useStash)
        return False

    def updateIndex(self, **kwargs):
        """Search and store PubChem correspondences for CCD and BIRD reference chemical definitions.

        Args:
            ccUrlTarget (str, optional): target url for chemical component dictionary resource file (default: None=all public)
            birdUrlTarget (str, optional): target url for bird dictionary resource file (cc format) (default: None=all public)
            ccFileNamePrefix (str, optional): index file prefix (default: full)
            rebuildChemIndices (bool, optional): rebuild indices from source (default: False)
            fetchLimit (int, optional): maximum number of definitions to process (default: None)
            exportPath(str, optional): path to export raw PubChem search results  (default: None)
            numProc(int):  number processors to include in multiprocessing mode (default: 12)

            Returns:
                (bool): True for success or False otherwise
        """
        ok = False
        try:
            rebuildChemIndices = kwargs.get("rebuildChemIndices", False)
            ccUrlTarget = kwargs.get("ccUrlTarget", None)
            birdUrlTarget = kwargs.get("birdUrlTarget", None)
            ccFileNamePrefix = kwargs.get("ccFileNamePrefix", "full")
            fetchLimit = kwargs.get("fetchLimit", None)
            exportPath = kwargs.get("exportPath", None)
            expireDays = kwargs.get("expireDays", 0)
            numProc = kwargs.get("numProc", 12)

            #  -- Update/create mapping index cache  ---
            ok = self.__pcicP.updateMissing(
                expireDays=expireDays,
                cachePath=self.__cachePath,
                ccUrlTarget=ccUrlTarget,
                birdUrlTarget=birdUrlTarget,
                ccFileNamePrefix=ccFileNamePrefix,
                exportPath=exportPath,
                rebuildChemIndices=rebuildChemIndices,
                fetchLimit=fetchLimit,
                numProc=numProc,
            )
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return ok

    def getMatches(self):
        """Return a list of matched PubChem compound identifiers.

        Returns:
            (list, str): list of PubChem compound identifiers
        """
        return self.__pcicP.getMatches()

    def getSelectedMatches(self, **kwargs):
        """
            Return preferred PubChem correspondences from the current match index for the input source
            component build type. Separately return alternative matches for other source types.

        Args:
            sourceTypes (list, optional):  list of source chemical component build types (default: ["model-xyz"])

        Returns:
            (dict, dict): mapD { ccId1: [{'pcId': ... , 'inchiKey': ... }], ccId2: ...},
                         altD { ccId1: [{'pcId': ... , 'inchiKey': ... 'sourceType': ... }], ccId2: ...}
        """
        sourceTypes = kwargs.get("sourceTypes", ["model-xyz"])
        mapD, extraMapD = self.__pcicP.getSelectedMatches(exportPath=self.__dirPath, sourceTypes=sourceTypes)
        logger.debug("mapD (%d) extraMapD (%d) %r", len(mapD), len(extraMapD), extraMapD)
        return mapD, extraMapD

    def updateData(self, pcidList, doExport=False):
        """Update PubChem reference data for the input list of compound identifiers.

        Args:
            pcidList (list,str): PubChem compound identifiers

        Returns:
            (bool): True for success or False otherwise
        """
        ok = False
        try:
            exportPath = self.__dirPath if doExport else None
            ok, failList = self.__pcdcP.updateMissing(pcidList, exportPath=exportPath)
            if failList:
                logger.info("No data updated for %r", failList)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return ok

    def updateMatchedData(self, exportRaw=False):
        """Update PubChem reference data using matched compound identifiers in the current index.

        Returns:
            (bool): True for success or False otherwise
        """
        ok = False
        try:
            pcidList = self.getMatches()
            exportPath = self.__dirPath if exportRaw else None
            ok, failList = self.__pcdcP.updateMissing(pcidList, exportPath=exportPath)
            if failList:
                logger.info("No data updated for %r", failList)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return ok

    def __getPubChemIdentifiers(self, pcidList):
        """Return related identifiers (xrefs) for the input PubChem compound identifier list.

        Args:
            pcidList (list): PubChem compound identifier list

        Returns:
            (dict) :{<pcid>: {'relatedId1': ... 'relatedId2': ... }, ...}

        """
        rD = self.__pcdcP.getRelatedMapping(pcidList)
        logger.info("Related identifier map length (%d)", len(rD))
        return rD

    def updateIdentifiers(self, **kwargs):
        """Update PubChem assigned related identifiers for matching compounds for the input chemical component sourceTypes.

        Args:
            sourceTypes (list, optional):  list of source chemical component build types (default: ["model-xyz"])

        Returns:
            (bool): True for success or False otherwise
        """
        ok = False
        try:
            sourceTypes = kwargs.get("sourceTypes", ["model-xyz"])
            mapD, _ = self.getSelectedMatches(sourceTypes=sourceTypes)
            pcIdList = []
            # mapD { ccId1: [{'pcId': ... , 'inchiKey': ... }],
            for mDL in mapD.values():
                pcIdList.extend([mD["pcId"] for mD in mDL])
            logger.info("pcIdList (%d)", len(pcIdList))
            rD = self.__getPubChemIdentifiers(pcIdList)
            #
            #  Update the identifier mappings
            for _, mDL in mapD.items():
                for mD in mDL:
                    pcId = mD["pcId"]
                    if pcId in rD:
                        for rIdName, rIdValue in rD[pcId].items():
                            mD[rIdName] = rIdValue
            #
            self.__identifierD = mapD
            ok = self.__identifierD is not None
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return ok

    def getIdentifiers(self, **kwargs):
        """Get PubChem assigned related identifiers for matching compounds for the input chemical component sourceTypes.

        Returns:
            dict: riD { ccId1: [{'pcId': ... , 'inchiKey': ... 'ChEBI': ... 'ChEMBL': ... 'CAS': ... }], ccId2: ...},

        """
        if not self.__identifierD:
            self.updateIdentifiers(**kwargs)
        return self.__identifierD

    def __dumpIdentifiers(self):
        rD = self.getIdentifiers()
        ok = self.__pcP.load(rD, "identifiers", fmt="json")
        return ok



##
# File:    ReferenceSequenceAnnotationAdapterTests.py
# Author:  J. Westbrook
# Date:    14-Feb-2020
#
# Updates:
#
##
"""
Tests of reference seequence annotation adapter.
"""

__docformat__ = "google en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"

import logging
import os
import platform
import resource
import time
import unittest

from rcsb.exdb.seq.ReferenceSequenceAnnotationAdapter import ReferenceSequenceAnnotationAdapter
from rcsb.exdb.seq.ReferenceSequenceAnnotationProvider import ReferenceSequenceAnnotationProvider
from rcsb.exdb.utils.ObjectTransformer import ObjectTransformer
from rcsb.utils.config.ConfigUtil import ConfigUtil

logging.basicConfig(level=logging.INFO, format="%(asctime)s [%(levelname)s]-%(module)s.%(funcName)s: %(message)s")
logger = logging.getLogger()

HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(os.path.dirname(HERE)))


class ReferenceSequenceAnnotationAdapterTests(unittest.TestCase):
    def __init__(self, methodName="runTest"):
        super(ReferenceSequenceAnnotationAdapterTests, self).__init__(methodName)
        self.__verbose = True

    def setUp(self):
        #
        self.__mockTopPath = os.path.join(TOPDIR, "rcsb", "mock-data")
        self.__cachePath = os.path.join(TOPDIR, "CACHE")
        self.__useCache = True
        configPath = os.path.join(TOPDIR, "rcsb", "mock-data", "config", "dbload-setup-example.yml")
        configName = "site_info_configuration"
        self.__cfgOb = ConfigUtil(configPath=configPath, defaultSectionName=configName, mockTopPath=self.__mockTopPath)
        #
        self.__resourceName = "MONGO_DB"
        self.__fetchLimit = None
        self.__startTime = time.time()
        logger.debug("Starting %s at %s", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()))

    def tearDown(self):
        unitS = "MB" if platform.system() == "Darwin" else "GB"
        rusageMax = resource.getrusage(resource.RUSAGE_SELF).ru_maxrss
        logger.info("Maximum resident memory size %.4f %s", rusageMax / 10 ** 6, unitS)
        endTime = time.time()
        logger.info("Completed %s at %s (%.4f seconds)", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()), endTime - self.__startTime)

    def testAnnotationAdapter(self):
        """Test case - create and read cache reference sequences assignments and related data."""
        try:
            databaseName = "pdbx_core"
            collectionName = "pdbx_core_polymer_entity"
            polymerType = "Protein"
            #  -- create cache ---
            rsaP = ReferenceSequenceAnnotationProvider(
                self.__cfgOb, databaseName, collectionName, polymerType, fetchLimit=self.__fetchLimit, siftsAbbreviated="TEST", cachePath=self.__cachePath, useCache=True
            )
            ok = rsaP.testCache()
            self.assertTrue(ok)
            numRef1 = rsaP.getRefDataCount()
            #
            # ---  Reload from cache ---
            rsaP = ReferenceSequenceAnnotationProvider(self.__cfgOb, databaseName, collectionName, polymerType, cachePath=self.__cachePath, useCache=True)
            ok = rsaP.testCache()
            self.assertTrue(ok)
            numRef2 = rsaP.getRefDataCount()
            self.assertEqual(numRef1, numRef2)
            #
            rsa = ReferenceSequenceAnnotationAdapter(rsaP)
            obTr = ObjectTransformer(self.__cfgOb, objectAdapter=rsa)
            ok = obTr.doTransform(
                databaseName=databaseName, collectionName=collectionName, fetchLimit=self.__fetchLimit, selectionQuery={"entity_poly.rcsb_entity_polymer_type": polymerType}
            )
            self.assertTrue(ok)

        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()


def referenceSequenceAnnotationAdapterSuite():
    suiteSelect = unittest.TestSuite()
    suiteSelect.addTest(ReferenceSequenceAnnotationAdapterTests("testAnnotationAdapter"))
    return suiteSelect


if __name__ == "__main__":
    mySuite = referenceSequenceAnnotationAdapterSuite()
    unittest.TextTestRunner(verbosity=2).run(mySuite)



##
# File: GlycanEtlWorkflow.py
# Date: 30-Jun-2021  jdw
#
#  Workflow wrapper  --  Glycan ETL utilities
#
#  Updates:
#
##
__docformat__ = "google en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"

import logging
import os


from rcsb.exdb.branch.GlycanProvider import GlycanProvider
from rcsb.utils.config.ConfigUtil import ConfigUtil

logger = logging.getLogger(__name__)


class GlycanEtlWorkflow(object):
    def __init__(self, **kwargs):
        """Workflow wrapper  --  Glycan mapping ETL utilities

        Args:
            configPath (str, optional): path to configuration file (default: exdb-config-example.yml)
            configName (str, optional): configuration section name (default: site_info_configuration)
            cachePath (str, optional):  path to cache directory (default: '.')
            stashRemotePrefix (str, optional): file name prefix (channel) applied to remote stash file artifacts (default: None)
        """
        configPath = kwargs.get("configPath", "exdb-config-example.yml")
        self.__configName = kwargs.get("configName", "site_info_configuration")
        mockTopPath = kwargs.get("mockTopPath", None)
        self.__cfgOb = ConfigUtil(configPath=configPath, defaultSectionName=self.__configName, mockTopPath=mockTopPath)
        #
        self.__cachePath = kwargs.get("cachePath", ".")
        self.__cachePath = os.path.abspath(self.__cachePath)
        self.__stashRemotePrefix = kwargs.get("stashRemotePrefix", None)
        #
        self.__debugFlag = kwargs.get("debugFlag", False)
        if self.__debugFlag:
            logger.setLevel(logging.DEBUG)
        #

    def updateMatchedIndex(self, backup=True):
        """Update Glycan correspondence index.

        Returns:
            (bool): True for success or False otherwise

        """
        try:
            ok1 = ok2 = False
            gP = GlycanProvider(cachePath=self.__cachePath, useCache=True)
            ok = gP.restore(self.__cfgOb, self.__configName, self.__stashRemotePrefix, useStash=True, useGit=True)
            logger.info("Restore glycan matched identifiers status (%r)", ok)

            ok1 = gP.update(self.__cfgOb, fmt="json", indent=3)
            riD = gP.getIdentifiers()
            logger.info("Matched glycan identifiers (%d)", len(riD))
            #
            if backup:
                ok2 = gP.backup(self.__cfgOb, self.__configName, self.__stashRemotePrefix, useGit=True, useStash=True)
                logger.info("Backup matched glycan identifiers (%r)", ok2)
            else:
                ok2 = True

        except Exception as e:
            logger.exception("Failing with %s", str(e))
        #
        return ok1 and ok2
